#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.ModelDefinition import ModelDefinition
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.settings.Settings import Settings

from re import match


class ListOfModelDefinitions(ListOf, HasIds):
	""" Class for the listOfModelDefinition in a sbml model """

	def __init__ (self, model):

		self.__model = model
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)

	def readSbml(self, sbml_external_models,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads compartments' list from a sbml file """

		for model in sbml_external_models:
			t_model = ModelDefinition(self.__model, self.nextId())
			t_model.readSbml(model, sbml_level, sbml_version)
			ListOf.add(self, t_model)

	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes compartments' list to a sbml file """

		for model in self:
			sbml_md = sbml_model.createModelDefinition()
			model.writeSbml(sbml_md, sbml_level, sbml_version)


	def new(self, name=None, sbml_id=None):
		""" Creates a new compartment """

		t_model = ModelDefinition(self.__model, self.nextId())
		ListOf.add(self, t_model)
		return t_model

	def remove(self, comp):
		""" Remove an object from the list """
		dict.__delitem__(self, comp.objId)

	def removeById(self, obj_id):
		""" Remove an object from the list """
		dict.__delitem__(self, obj_id)

	def getListOfModelDefinitions(self):
		res = []
		for internal_model in self:
			res.append(internal_model)
		return res

	def resolveXPath(self, selector):

		if selector.startswith("modelDefinition") or selector.startswith("sbml:modelDefinition"):

			res_match = match(r'(.+)\[@(.+)=\'(.+)\'\]', selector)
			if res_match is not None:
				tokens = res_match.groups()

				if len(tokens) == 3:
					if tokens[1] == "id":
						return self.getBySbmlId(tokens[2])
					elif tokens[1] == "name":
						return self.getByName(tokens[2])

		# If not returned yet
		raise InvalidXPath(selector)


	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, value):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], value)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from PyDSTool import args, Generator
from PyDSTool.Toolbox import phaseplane as pp

from libsignetsim.model.math.sympy_shortcuts import SympySymbol
from pprint import pformat


class PyDSToolModel(object):

	def __init__ (self, model):

		self.model = model

		self.systemParameters = None
		self.system = None

	def getSystem(self):
		return self.system

	def build(self, parameter, from_value, vars_to_keep=[]):

		parameter.setValue(from_value)
		self.model.build(vars_to_keep=vars_to_keep, reduce=True)
		self.buildDS()

	def buildDS(self):

		comp_subs = {}
		for comp in list(self.model.listOfCompartments):
			comp_subs.update({comp.symbol.getInternalMathFormula():comp.value.getInternalMathFormula()})

		parameters = {}
		variables = {}
		odes = {}

		subs_cfes = {}
		for cfe in self.model.getMathModel().listOfCFEs:
			t_definition = cfe.getDefinition().getDeveloppedInternalMathFormula()
			subs_cfes.update({cfe.getVariable().symbol.getInternalMathFormula(): t_definition})

		for variable in list(self.model.getMathModel().listOfVariables):

			if variable.isConstant():
				t_symbol = variable.symbol.getInternalMathFormula()
				t_value = self.model.listOfInitialConditions[t_symbol].getInternalMathFormula()
				parameters.update({str(t_symbol): str(t_value)})

			elif variable.isDerivative():
				t_symbol = variable.symbol.getInternalMathFormula()
				t_value = self.model.listOfInitialConditions[t_symbol].getInternalMathFormula()
				variables.update({str(t_symbol): str(t_value)})

		for ode in self.model.getMathModel().listOfODEs:
			t_ode = ode.getDefinition().getDeveloppedInternalMathFormula().subs(subs_cfes).subs(comp_subs)
			odes.update({str(ode.getVariable().symbol.getInternalMathFormula()): str(t_ode)})

		self.systemParameters = args(name=self.model.getSbmlId())
		self.systemParameters.pars = parameters
		self.systemParameters.varspecs = odes
		self.systemParameters.ics = variables
		self.system = Generator.Vode_ODEsystem(self.systemParameters)

	def __str__(self):

		res = "\n" + pformat(self.systemParameters.pars)
		res += "\n" + pformat(self.systemParameters.ics)
		res += "\n" + pformat(self.systemParameters.varspecs)
		res += "\n"

		return res


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings


class SubTask(object):

	def __init__(self, document):

		self.__document = document
		self.__task = None
		self.__order = None

	def readSedml(self, subtask, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		if subtask.isSetTask():
			self.__task = subtask.getTask()

		if subtask.isSetOrder():
			self.__order = subtask.getOrder()

	def writeSedml(self, subtask, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		if self.__task is not None:
			subtask.setTask(self.__task)

		if self.__order is not None:
			subtask.setOrder(self.__order)

	def getTaskReference(self):
		return self.__task

	def getTask(self):
		return self.__document.listOfTasks.getTask(self.__task)

	def getOrder(self):
		return self.__order

	def setTaskReference(self, task_reference):
		self.__task = task_reference

	def setTask(self, task):
		self.__task = task.getId()

	def setOrder(self, order):
		self.__order = order




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

class ListOf(list):
	""" Parent class for all the ListOf.* containers in a sbml model """

	def __init__(self, model=None):

		list.__init__(self)
		self.__model = model
		self.isListOfSbmlObjects = False
		self.currentObjId = -1

	def nextId(self):
		self.currentObjId += 1
		return self.currentObjId

	def add(self, sbml_object):
		list.append(self, sbml_object)

	def index(self, object):
		return list.index(self, object)

	def ids(self):
		""" Return a set of ids of the sbml objects """
		return [obj.objId for obj in self]

	def getById(self, obj_id, pos=0):
		""" Find sbml objects by their import Id """

		res = []
		for obj in self:
			if obj.objId == obj_id:
				res.append(obj)

		if len(res) > 0:
			return res[pos]
		else:
			return None

	def getByPos(self, pos):
		""" Find sbml objects by their position """
		return list.__getitem__(self, pos)

	def getPosById(self, id):
		return self.index(self.getById(id))

	def remove(self, sbml_obj, full_remove=True):
		""" Remove an object from the list """

		if not self.isListOfSbmlObjects and full_remove:
			self.__model.listOfSbmlObjects.remove(sbml_obj)
		list.remove(self, sbml_obj)

	def removeById(self, obj_id):
		""" Remove an object from the list """
		self.remove(self.getById(obj_id))

	def clear(self):
		self.currentObjId = -1
		list.__init__(self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings

class SedBase(object):

	def __init__(self, document):

		self.__document = document
		self.__metaId = None
		self.__notes = None
		self.__annotations = None

	def readSedml(self, sed_base_object, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		if sed_base_object.isSetMetaId():
			self.__metaId = sed_base_object.getMetaId()

		if sed_base_object.isSetNotes():
			self.__notes = sed_base_object.getNotes()

		if sed_base_object.isSetAnnotation():
			self.__annotations = sed_base_object.getAnnotation()

	def writeSedml(self, sed_base_object, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		if self.__metaId is not None:
			sed_base_object.setMetaId(self.__metaId)

		if self.__notes is not None:
			sed_base_object.setNotes(self.__notes)

		if self.__annotations is not None:
			sed_base_object.setAnnotation(self.__annotations)


	def getMetaId(self):
		return self.__metaId

	def getNotes(self):
		return self.__notes

	def setMetaId(self, meta_id):
		self.__metaId = meta_id

	def setNotes(self, notes):
		self.__notes = notes


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the module URIs

"""




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the combine archive module

"""

from os.path import dirname, basename, join
from mimetypes import guess_type
from lxml import etree
from libsbml import SBMLReader
import libsbml
from libnuml import readNUMLFromString
from libsedml import readSedMLFromString
from six.moves import reload_module
reload_module(libsbml)


class File(object):

	XML = "http://purl.org/NET/mediatypes/application/xml"
	SBML = "http://identifiers.org/combine.specifications/sbml"
	SEDML = "http://identifiers.org/combine.specifications/sed-ml"
	NUML = "http://identifiers.org/combine.specifications/numl"
	CELLML = "http://identifiers.org/combine.specifications/cellml"
	MANIFEST = "http://identifiers.org/combine.specifications/omex-manifest"
	PDF = "http://purl.org/NET/mediatypes/application/pdf"
	MARKDOWN = "http://purl.org/NET/mediatypes/text/markdown"
	UNKNOWN = "http://purl.org/NET/mediatypes/application/octet-stream"


	KNOWN_FORMATS = {
		"xml" : XML,
		"sbml" : SBML,
		"sedml" : SEDML,
		"numl" : NUML,
		"cellml" : CELLML,
		"pdf" : PDF,
		"md" : MARKDOWN,
	}

	def __init__(self, archive, manifest):

		self.__archive = archive
		self.__manifest = manifest
		self.__fullFilename = None
		self.__content = None

		self.__path = None
		self.__filename = None
		self.__extension = None
		self.__format = None

	def readFile(self, filename, archive_file):

		self.__path = dirname(filename)
		self.__filename = basename(filename)
		self.__extension = self.__filename.split(".")[-1]

		if self.__manifest.isInManifest(self.getFilename()) and self.__manifest.getFormat(self.getFilename()) is not None:
			self.__format = self.__manifest.getFormat(self.getFilename())

		elif self.__extension in self.KNOWN_FORMATS:

			self.__format = self.KNOWN_FORMATS[self.__extension]

			if self.__format == self.XML:
				self.__format = self.guessXML(archive_file.read(filename))

		else:
			self.__format = "http://purl.org/NET/mediatypes/%s" % guess_type(filename)[0]

		if self.__format is None:
			self.__format = self.UNKNOWN


		if not self.__manifest.isInManifest(self.getFilename()):
			self.__manifest.addInManifest(self.getFilename(), self.__format)

		elif self.__manifest.getFormat(self.getFilename()) is None:
			self.__manifest.setFormat(self.getFilename(), self.__format)

	def newFile(self, filename):

		self.__filename = basename(filename)
		self.__extension = self.__filename.split('.')[-1]

		if self.__extension in self.KNOWN_FORMATS:
			self.__format = self.KNOWN_FORMATS[self.__extension]
			if self.__format == self.XML:
				with open(filename, 'rb') as new_file:
					self.__format = self.guessXML(new_file.read())

		else:
			self.__format = "http://purl.org/NET/mediatypes/%s" % guess_type(filename)[0]

		if self.__format is None:
			self.__format = self.UNKNOWN

		self.__manifest.addInManifest(self.__filename, self.__format)


	def isMaster(self):
		return self.__manifest.isInManifest(self.getFilename()) and self.__manifest.isMaster(self.getFilename())

	def setMaster(self):
		if self.__manifest.isInManifest(self.getFilename()):
			self.__manifest.setMaster(self.getFilename())

	def isSedml(self):
		return (self.__manifest.isInManifest(self.getFilename())
				and self.__manifest.getFormat(self.getFilename()).startswith(self.SEDML)
		)

	def isSbml(self):
		return (self.__manifest.isInManifest(self.getFilename())
				and self.__manifest.getFormat(self.getFilename()).startswith(self.SBML)
		)

	def isNuml(self):
		return (
			self.__manifest.isInManifest(self.getFilename())
			and self.__manifest.getFormat(self.getFilename()).startswith(self.NUML)
		)

	def getFilename(self):
		if self.__path is not None:
			return join(self.__path, self.__filename)
		else:
			return self.__filename

	def getFormat(self):
		return self.__format

	def guessXML(self, xml_content):

		root = etree.fromstring(xml_content)
		tag = root.tag.split("}")[1]

		if isinstance(xml_content, bytes):
			xml_content = str("%s" % xml_content.decode('ascii', 'ignore'))

		if tag == "sbml":
			sbmlReader = SBMLReader()
			if sbmlReader is not None:
				sbmlDoc = sbmlReader.readSBMLFromString(xml_content)
				return self.SBML + ".level-%d.version-%d" % (sbmlDoc.getLevel(), sbmlDoc.getVersion())
			else:
				return self.SBML

		elif tag == "sedML":
			sedmlDoc = readSedMLFromString(xml_content)
			return self.SEDML + ".level-%d.version-%d" % (sedmlDoc.getLevel(), sedmlDoc.getVersion())

		elif tag == "numl":
			numlDoc = readNUMLFromString(xml_content)
			return self.NUML + ".level-%d.version-%d" % (numlDoc.getLevel(), numlDoc.getVersion())

		elif tag == "omexManifest":
			return self.MANIFEST

		else:
			return self.XML





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.sbml.container.ListOfSpecies import ListOfSpecies
from libsignetsim.model.sbml.container.ListOfParameters import ListOfParameters
from libsignetsim.model.sbml.container.ListOfReactions import ListOfReactions
from libsignetsim.model.sbml.container.ListOfCompartments import ListOfCompartments
from libsignetsim.model.sbml.container.ListOfFunctionDefinitions import ListOfFunctionDefinitions
from libsignetsim.model.sbml.container.ListOfUnitDefinitions import ListOfUnitDefinitions
from libsignetsim.model.sbml.container.ListOfRules import ListOfRules
from libsignetsim.model.sbml.container.ListOfEvents import ListOfEvents
from libsignetsim.model.sbml.container.ListOfConstraints import ListOfConstraints
from libsignetsim.model.sbml.container.ListOfInitialAssignments import ListOfInitialAssignments
from libsignetsim.model.sbml.container.ListOfSbmlObjects import ListOfSbmlObjects
from libsignetsim.model.sbml.container.ListOfSubmodels import ListOfSubmodels
from libsignetsim.model.sbml.container.ListOfPorts import ListOfPorts

from libsignetsim.settings.Settings import Settings
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.ModelUnits import ModelUnits
from libsignetsim.model.sbml.SbmlModelAnnotation import SbmlModelAnnotation
from libsignetsim.model.sbml.HasId import HasId
from libsignetsim.model.sbml.HasConversionFactor import HasConversionFactor
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.ModelException import InvalidXPath

from time import time
from re import match


class SbmlModel(HasId, SbmlObject, ModelUnits, SbmlModelAnnotation, HasConversionFactor, HasParentObj):
	""" Sbml model class """


	def __init__ (self, parent_document, parent_obj, obj_id=0):
		""" Constructor of model class """

		self.objId = obj_id

		self.sbmlLevel = Settings.defaultSbmlLevel
		self.sbmlVersion = Settings.defaultSbmlVersion

		if parent_document is None:
			from libsignetsim.model.SbmlDocument import SbmlDocument
			self.parentDoc = SbmlDocument(self)
		else:
			self.parentDoc = parent_document

		if parent_obj is None:
			HasParentObj.__init__(self, self.parentDoc)
		else:
			HasParentObj.__init__(self, parent_obj)

		HasId.__init__(self, self)
		self.listOfSbmlObjects = ListOfSbmlObjects(self)
		SbmlObject.__init__(self, self)
		SbmlModelAnnotation.__init__(self)
		ModelUnits.__init__(self)
		HasConversionFactor.__init__(self, self)

		self.listOfSpecies = ListOfSpecies(self, self)
		self.listOfParameters = ListOfParameters(self, self)
		self.listOfReactions = ListOfReactions(self, self)
		self.listOfCompartments = ListOfCompartments(self, self)
		self.listOfFunctionDefinitions = ListOfFunctionDefinitions(self)
		self.listOfUnitDefinitions = ListOfUnitDefinitions(self)
		self.listOfRules = ListOfRules(self)
		self.listOfEvents = ListOfEvents(self, self)
		self.listOfConstraints = ListOfConstraints(self)
		self.listOfInitialAssignments = ListOfInitialAssignments(self, self)

		self.listOfSubmodels = ListOfSubmodels(self)
		self.listOfPorts = ListOfPorts(self)

		self.defaultCompartment = None

	def newModel(self, name):

		self.setName(name)
		self.setSbmlId("model_%d" % self.objId)

		ModelUnits.setDefaultUnits(self)
		self.defaultCompartment = self.listOfCompartments.new("cell")

	def readSbml(self, sbmlModel,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion, parent_doc=None):

		t0 = time()

		self.sbmlLevel = sbml_level
		self.sbmlVersion = sbml_version

		HasId.readSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)
		SbmlObject.readSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)
		SbmlModelAnnotation.readSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)

		self.listOfFunctionDefinitions.readSbml(sbmlModel.getListOfFunctionDefinitions(), self.sbmlLevel, self.sbmlVersion)
		self.listOfUnitDefinitions.readSbml(sbmlModel.getListOfUnitDefinitions(), self.sbmlLevel, self.sbmlVersion)
		ModelUnits.readSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)


		self.listOfCompartments.readSbml(sbmlModel.getListOfCompartments(), self.sbmlLevel, self.sbmlVersion)
		self.listOfParameters.readSbml(sbmlModel.getListOfParameters(), self.sbmlLevel, self.sbmlVersion)

		# We need to load the parameters before reading the conversion factor
		if self.sbmlLevel >= 3:
			if sbmlModel.isSetConversionFactor():
				HasConversionFactor.readSbml(self, sbmlModel.getConversionFactor(), self.sbmlLevel, self.sbmlVersion)

		self.listOfSpecies.readSbml(sbmlModel.getListOfSpecies(), self.sbmlLevel, self.sbmlVersion)
		self.listOfReactions.readSbml(sbmlModel.getListOfReactions(), self.sbmlLevel, self.sbmlVersion)
		self.listOfInitialAssignments.readSbml(sbmlModel.getListOfInitialAssignments(), self.sbmlLevel, self.sbmlVersion)
		self.listOfRules.readSbml(sbmlModel.getListOfRules(), self.sbmlLevel, self.sbmlVersion)
		self.listOfConstraints.readSbml(sbmlModel.getListOfConstraints(), self.sbmlLevel, self.sbmlVersion)
		self.listOfEvents.readSbml(sbmlModel.getListOfEvents(), self.sbmlLevel, self.sbmlVersion)

		if self.sbmlLevel == 3 and self.parentDoc.isCompEnabled():
			self.listOfSubmodels.readSbml(sbmlModel.getPlugin("comp").getListOfSubmodels(), self.sbmlLevel, self.sbmlVersion)
			self.listOfPorts.readSbml(sbmlModel.getPlugin("comp").getListOfPorts(), self.sbmlLevel, self.sbmlVersion)

		if Settings.verboseTiming >= 1:
			print(">> SBML Model %s read in %.2gs" % (self.getSbmlId(), time()-t0))


	def writeSbml(self, sbmlModel,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		t0 = time()

		if self.sbmlLevel == 3 and self.parentDoc.isCompEnabled():
			sbmlModel.enablePackage("http://www.sbml.org/sbml/level3/version1/comp/version1", "comp", True)

		SbmlObject.writeSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)
		HasId.writeSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)
		SbmlModelAnnotation.writeSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)

		self.listOfUnitDefinitions.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfFunctionDefinitions.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfCompartments.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfSpecies.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfParameters.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfInitialAssignments.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfRules.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfConstraints.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfReactions.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		self.listOfEvents.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)
		ModelUnits.writeSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)

		if self.sbmlLevel >= 3:
			if self.isSetConversionFactor():
				HasConversionFactor.writeSbml(self, sbmlModel, self.sbmlLevel, self.sbmlVersion)

		if self.sbmlLevel == 3 and self.parentDoc.isCompEnabled():
			self.listOfSubmodels.writeSbml(sbmlModel.getPlugin("comp"), self.sbmlLevel, self.sbmlVersion)
			self.listOfPorts.writeSbml(sbmlModel.getPlugin("comp"), self.sbmlLevel, self.sbmlVersion)

		if Settings.verboseTiming >= 2:
			print("> SBML Model %s written in %.2gs" % (self.getSbmlId(), time()-t0))

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		""" Here we rename the variable in all the math """

		self.listOfSpecies.renameSbmlId(old_sbml_id, new_sbml_id)
		self.listOfVariables.renameSbmlId(old_sbml_id, new_sbml_id)
		self.listOfInitialAssignments.renameSbmlId(old_sbml_id, new_sbml_id)
		self.listOfRules.renameSbmlId(old_sbml_id, new_sbml_id)
		self.listOfReactions.renameSbmlId(old_sbml_id, new_sbml_id)
		self.listOfEvents.renameSbmlId(old_sbml_id, new_sbml_id)


	def getSbmlLevels(self):
		return [1,2,3]

	def getSbmlVersions(self):
		if self.sbmlLevel == 1:
			return [2]
		elif self.sbmlLevel == 2:
			return [1,2,3,4,5]
		elif self.sbmlLevel == 3:
			return [1]
		else:
			return []


	def setSbmlLevel(self, level):

		self.sbmlLevel = level

		if level == 1:
			self.sbmlVersion = 2
		elif level == 2:
			self.sbmlVersion = 5
		elif level == 3:
			self.sbmlVersion = 1

	def resolveXPath(self, selector):

		if selector in ["sbml:listOfSpecies", "listOfSpecies"]:
			return self.listOfSpecies

		elif selector in ["sbml:listOfParameters", "listOfParameters"]:
			return self.listOfParameters

		elif selector in ["sbml:listOfCompartments", "listOfCompartments"]:
			return self.listOfCompartments

		elif selector in ["sbml:listOfReactions", "listOfReactions"]:
			return self.listOfReactions

		elif selector in ["sbml:listOfInitialAssignments", "listOfInitialAssignments"]:
			return self.listOfInitialAssignments

		elif selector in ["sbml:listOfEvents", "listOfEvents"]:
			return self.listOfEvents


		elif selector == "sbml:listOfSubmodels" and self.parentDoc.useCompPackage:
			return self.listOfSubmodels


		elif selector.startswith("descendant::"):
			res_match = match(r"descendant::\*\[@(.*)=\'(.*)\'\]", selector)
			if res_match is None:
				raise InvalidXPath(selector)

			res = res_match.groups()
			if res[0] == "id":
				return self.listOfVariables.getBySbmlId(res[1])

			elif res[0] == "name":
				return self.listOfVariables.getByName(res[1])

			elif res[0] == "metaid":
				return self.listOfSbmlObjects.getByMetaId(res[1])

		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)

	def getXPath(self):
		if self.getParentObj() is not None:
			return "/".join([self.getParentObj().getXPath(), "sbml:model"])
		else:
			return "sbml:model"


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""


"""

from libsignetsim.model.sbml.HasId import HasId
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.sbml.container.ListOfDeletions import ListOfDeletions
from libsignetsim.model.ModelException import UnknownSubmodelRefException


class SubModel(HasId):

	def __init__(self, model, obj_id):

		self.__model = model
		self.objId = obj_id

		HasId.__init__(self, model)

		self.__modelRef = None
		self.__timeConversionFactor = None
		self.__extentConversionFactor = None
		self.listOfDeletions = ListOfDeletions(model, self)

	def readSbml(self, sbml_submodel,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		HasId.readSbml(self, sbml_submodel, sbml_level, sbml_version)

		if sbml_submodel.isSetModelRef():
			self.__modelRef = sbml_submodel.getModelRef()

		if sbml_submodel.isSetTimeConversionFactor():
			self.__timeConversionFactor = MathFormula(self.__model)
			self.__timeConversionFactor.readSbml(
					sbml_submodel.getTimeConversionFactor(),
					sbml_level, sbml_version)

		if sbml_submodel.isSetExtentConversionFactor():
			self.__extentConversionFactor = MathFormula(self.__model)
			self.__extentConversionFactor.readSbml(
					sbml_submodel.getExtentConversionFactor(),
					sbml_level, sbml_version)

		self.listOfDeletions.readSbml(sbml_submodel.getListOfDeletions(),
									sbml_level, sbml_version)



	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		sbml_submodel = sbml_model.createSubmodel()

		HasId.writeSbml(self, sbml_submodel, sbml_level, sbml_version)
		if self.__modelRef is not None:
			sbml_submodel.setModelRef(self.__modelRef)

		if self.__timeConversionFactor is not None:
			sbml_submodel.setTimeConversionFactor(self.__timeConversionFactor.getSbmlMathFormula().getName())

		if self.__extentConversionFactor is not None:
			sbml_submodel.setExtentConversionFactor(self.__extentConversionFactor.getSbmlMathFormula().getName())

		self.listOfDeletions.writeSbml(sbml_submodel, sbml_level, sbml_version)

	def hasExtentConversionFactor(self):
		return self.__extentConversionFactor is not None

	def getExtentConversionFactor(self):
		return self.__extentConversionFactor

	def setExtentConversionFactor(self, conv_factor):
		if self.__extentConversionFactor is None:
			self.__extentConversionFactor = MathFormula(self.__model)
		self.__extentConversionFactor.setPrettyPrintMathFormula(conv_factor)

	def unsetExtentConversionFactor(self):
		self.__extentConversionFactor = None

	def hasTimeConversionFactor(self):
		return self.__timeConversionFactor is not None

	def getTimeConversionFactor(self):
		return self.__timeConversionFactor

	def setTimeConversionFactor(self, conv_factor):
		if self.__timeConversionFactor is None:
			self.__timeConversionFactor = MathFormula(self.__model)
		self.__timeConversionFactor.setPrettyPrintMathFormula(conv_factor)

	def unsetTimeConversionFactor(self):
		self.__timeConversionFactor = None

	def setModelRef(self, model_ref):
		self.__modelRef = model_ref

	def getModelRef(self):
		return self.__modelRef

	def getModelObject(self):

		if self.__modelRef == self.__model.getSbmlId():
			return self.__model
		else:
			t_doc = self.__model.parentDoc
			if self.__modelRef in t_doc.listOfModelDefinitions.sbmlIds():
				return t_doc.listOfModelDefinitions.getBySbmlId(self.__modelRef)
			elif self.__modelRef in t_doc.listOfExternalModelDefinitions.sbmlIds():
				return t_doc.listOfExternalModelDefinitions.getBySbmlId(self.__modelRef).modelDefinition
			else:
				raise UnknownSubmodelRefException("Model reference %s not found !" % self.__modelRef)

	def getModelInstance(self):
		from libsignetsim.model.ModelInstance import ModelInstance
		return ModelInstance(self.getModelObject(), self.__model.parentDoc)

	def getByXPath(self, xpath):
		return self.getModelObject().getByXPath(xpath)

	def setByXPath(self, xpath, object):
		self.getModelObject().setByXPath(xpath, object)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""


"""

from libsignetsim.settings.Settings import Settings
from libsignetsim.model.sbml.HasRef import HasRef

class ReplacedBy(HasRef):

	def __init__(self, model, parent_obj):

		self.__model = model
		self.__parentObj = parent_obj
		HasRef.__init__(self, model)
		self.__submodelRef = None



	def readSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		HasRef.readSbml(self, sbml_object, sbml_level, sbml_version)
		if sbml_object.isSetSubmodelRef():
			self.__submodelRef = sbml_object.getSubmodelRef()


	def writeSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		sbml_rb = sbml_object.createReplacedBy()
		HasRef.writeSbml(self, sbml_rb, sbml_level, sbml_version)
		if self.__submodelRef is not None:
			sbml_rb.setSubmodelRef(self.__submodelRef)

	def setSubmodelRef(self, submodel_ref):
		self.__submodelRef = submodel_ref


	def getSubmodelRef(self):
		return self.__submodelRef

	def hasModelRef(self):
		return self.__submodelRef is not None

	def getReplacingElementObjectFromInstance(self, model_instance):

		# Now choosing the right model
		if self.hasModelRef():

			if self.getSubmodelRef() == self.__model.getSbmlId():
				tt_model = self.__model
				tt_instance = model_instance

			else:
				tt_model = self.__model.listOfSubmodels.getBySbmlId(self.getSubmodelRef()).getModelObject()
				tt_instance = model_instance.getSubmodelInstance(self.getSubmodelRef())

			if self.hasIdRef():

				if self.hasSBaseRef():

					ttt_model = tt_model.listOfSubmodels.getBySbmlId(self.getIdRef()).getModelObject()
					ttt_instance = tt_instance.getSubmodelInstance(self.getIdRef())
					refs = self.getSBaseRef().getRef(ttt_model)
					obj_dicts = [ttt_instance.objectsDictionnary]
					while len(refs) > 1:
						ttt_model = ttt_model.listOfSubmodels.getBySbmlId(refs[0]).getModelObject()
						ttt_instance = ttt_instance.getSubmodelInstance(refs[0])
						obj_dicts.append(ttt_instance.objectsDictionnary)
						refs = refs[-1:]

					t_object = ttt_model.listOfSbmlObjects.getByMetaId(refs[0])
					ttt_metaid = ttt_instance.objectsDictionnary[t_object.getMetaId()]
					obj_dicts.pop()

					while len(obj_dicts) > 0:
						ttt_metaid = obj_dicts[-1][ttt_metaid]
						obj_dicts.pop()

					tt_metaid = tt_instance.objectsDictionnary[ttt_metaid]

					return tt_instance.listOfSbmlObjects.getByMetaId(tt_metaid)
				else:
					t_object = tt_model.listOfVariables.getBySbmlId(self.getIdRef())


			elif self.hasPortRef():
				t_object = tt_model.listOfPorts.getBySbmlId(self.getPortRef()).getRefObject()

			elif self.hasMetaId():
				t_object = tt_model.listOfSbmlObjects.getByMetaId(self.getMetaIdRef())

			return tt_instance.listOfSbmlObjects.getByMetaId(
				tt_instance.objectsDictionnary[t_object.getMetaId()]
			)

	def getReplacingElementObject(self):

		# Now choosing the right model
		if self.hasModelRef():

			if self.getSubmodelRef() == self.__model.getSbmlId():
				tt_model = self.__model

			else:
				tt_model = self.__model.listOfSubmodels.getBySbmlId(self.getSubmodelRef()).getModelObject()

			if self.hasIdRef():

				if self.hasSBaseRef():

					ttt_model = tt_model.listOfSubmodels.getBySbmlId(self.getIdRef()).getModelObject()
					refs = self.getSBaseRef().getRef(ttt_model)

					while len(refs) > 1:
						ttt_model = ttt_model.listOfSubmodels.getBySbmlId(refs[0]).getModelObject()
						refs = refs[-1:]

					return ttt_model.listOfSbmlObjects.getByMetaId(refs[0])

				else:
					return tt_model.listOfVariables.getBySbmlId(self.getIdRef())


			elif self.hasPortRef():
				return tt_model.listOfPorts.getBySbmlId(self.getPortRef()).getRefObject()

			elif self.hasMetaId():
				return tt_model.listOfSbmlObjects.getByMetaId(self.getMetaIdRef())


	def renameSbmlId(self, old_sbml_id, new_sbml_id, submodel_ref=None):

		if self.hasIdRef() and self.getIdRef() == old_sbml_id:
			self.setIdRef(new_sbml_id)

	def getParentObject(self):
		return self.__parentObj


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.UniformTimeCourse import UniformTimeCourse
from libsignetsim.sedml.SteadyState import SteadyState
from libsignetsim.sedml.OneStep import OneStep

from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import SEDML_SIMULATION_UNIFORMTIMECOURSE, SEDML_SIMULATION_ONESTEP, SEDML_SIMULATION_STEADYSTATE
from six.moves import reload_module
reload_module(libsbml)

class ListOfSimulations(ListOf):

	def __init__(self, document):

		ListOf.__init__(self, document)
		self.__document = document
		self.__simulationCounter = 0

	def new(self, simulation_type, simulation_id=None):

		t_simulation_id = simulation_id
		if t_simulation_id is None:
			t_simulation_id = "simulation_%d" % self.__simulationCounter

		if simulation_type == SEDML_SIMULATION_UNIFORMTIMECOURSE:
			simulation = UniformTimeCourse(self.__document)
			simulation.setId(t_simulation_id)
			ListOf.append(self, simulation)
			self.__simulationCounter += 1
			return simulation

		elif simulation_type == SEDML_SIMULATION_STEADYSTATE:
			simulation = SteadyState(self.__document)
			simulation.setId(t_simulation_id)
			ListOf.append(self, simulation)
			self.__simulationCounter += 1
			return simulation

		elif simulation_type == SEDML_SIMULATION_ONESTEP:
			simulation = OneStep(self.__document)
			simulation.setId(t_simulation_id)
			ListOf.append(self, simulation)
			self.__simulationCounter += 1
			return simulation

	def createUniformTimeCourse(self, simulation_id=None):
		return self.new(SEDML_SIMULATION_UNIFORMTIMECOURSE, simulation_id)

	def createSteadyState(self, simulation_id=None):
		return self.new(SEDML_SIMULATION_STEADYSTATE, simulation_id)

	def createOneStep(self, simulation_id=None):
		return self.new(SEDML_SIMULATION_ONESTEP, simulation_id)

	def readSedml(self, list_of_simulations, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_simulations, level, version)

		for t_simulation in list_of_simulations:

			if t_simulation.getTypeCode() == SEDML_SIMULATION_UNIFORMTIMECOURSE:
				simulation = UniformTimeCourse(self.__document)
				simulation.readSedml(t_simulation, level, version)
				ListOf.append(self, simulation)
				self.__simulationCounter += 1

			elif t_simulation.getTypeCode() == SEDML_SIMULATION_STEADYSTATE:
				simulation = SteadyState(self.__document)
				simulation.readSedml(t_simulation, level, version)
				ListOf.append(self, simulation)
				self.__simulationCounter += 1

			elif t_simulation.getTypeCode() == SEDML_SIMULATION_ONESTEP:
				simulation = OneStep(self.__document)
				simulation.readSedml(t_simulation, level, version)
				ListOf.append(self, simulation)
				self.__simulationCounter += 1

	def writeSedml(self, list_of_simulations, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_simulations, level, version)

		for t_simulation in self:

			if isinstance(t_simulation, UniformTimeCourse):
				simulation = list_of_simulations.createUniformTimeCourse()
				t_simulation.writeSedml(simulation, level, version)
				self.__simulationCounter += 1

			elif isinstance(t_simulation, SteadyState):
				simulation = list_of_simulations.createSteadyState()
				t_simulation.writeSedml(simulation, level, version)
				self.__simulationCounter += 1

			elif isinstance(t_simulation, OneStep):
				simulation = list_of_simulations.createOneStep()
				t_simulation.writeSedml(simulation, level, version)
				self.__simulationCounter += 1

	def getSimulation(self, simulation_reference):

		for simulation in self:
			if simulation.getId() == simulation_reference:
				return simulation
	#
	# def buildSimulation(self, simulation_reference, model):
	#
	# 	return self.getSimulation(simulation_reference).build(model)
	#
	# def runSimulation(self, simulation_reference):
	#
	# 	return self.getSimulation(simulation_reference).run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings
from libsignetsim.LibSigNetSimException import UnknownObservationException, UnknownTreatmentException, NoTreatmentException, NoObservationException


class CWriterData(object):

	def __init__(self, listOfExperiments=None, mapping=None, workingModel=None, interpolate=False, subdir=None, hasTreatments=False, hasObservations=False):

		self.mapping = mapping
		self.listOfExperiments = listOfExperiments
		self.workingModel = workingModel
		self.subdir = subdir
		self.hasTreatments = hasTreatments
		self.hasObservations = hasObservations

		self.checkListOfExperiments()

	def checkListOfExperiments(self):

		if self.listOfExperiments is not None and len(self.listOfExperiments) > 0:
			for i, experiment in enumerate(self.listOfExperiments):
				for j, condition in enumerate(experiment.listOfConditions.values()):

					# Here we need to find all the timings of the treatments. That will be the set of initial values.
					# Then, each set of initial values can have multiple assignments
					t_times = []
					for initial_value in list(condition.listOfInitialConditions.values()):
						t_times.append(initial_value.t)

					# Removing doublons
					t_times = list(set(t_times))


					nb_treatments = 0
					nb_observable = 0

					for k, t_time in enumerate(t_times):

						treatments = []
						for initial_value in list(condition.listOfInitialConditions.values()):
							if initial_value.t == t_time:
								treatments.append(initial_value)

						for l, treatment in enumerate(treatments):

							t_variable = None

							if self.mapping is not None and len(self.mapping) > 0:
								if i < len(self.mapping) and treatment.name in list(self.mapping[i].keys()):
									t_variable = self.workingModel.parentDoc.getByXPath(
										self.mapping[i][treatment.name],
										instance=self.workingModel.parentDoc.useCompPackage
									)
								# else:
								# 	raise UnknownTreatmentException(
								# 		"Variable %s not found in mapping" % treatment.name)

							elif treatment.name_attribute == "name":
								if self.workingModel.listOfVariables.containsName(treatment.name):
									t_variable = self.workingModel.listOfVariables.getByName(treatment.name)
								else:
									raise UnknownTreatmentException("Cannot find a variable called %s" % treatment.name)

							elif treatment.name_attribute == "id":
								if self.workingModel.listOfVariables.containsSbmlId(treatment.name):
									t_variable = self.workingModel.listOfVariables.getBySbmlId(treatment.name)
								else:
									raise UnknownTreatmentException("Cannot find a variable called %s" % treatment.name)

							else:
								raise UnknownTreatmentException(
									"Unknown attribute for variable : %s" % treatment.name_attribute)

							if t_variable is not None:
								nb_treatments += 1

						# Observed_values
					vars_observed = {}

					for k, observed_value in enumerate(condition.listOfExperimentalData.values()):

						t_variable = None
						if self.mapping is not None and len(self.mapping) > 0:
							if i < len(self.mapping) and observed_value.name in list(self.mapping[i].keys()):
								t_variable = self.workingModel.parentDoc.getByXPath(
									self.mapping[i][observed_value.name],
									instance=self.workingModel.parentDoc.useCompPackage
								)
							# else:
							# 	raise UnknownObservationException("Variable %s not found in mapping" % observed_value.name)

						elif observed_value.name_attribute == "name":
							if self.workingModel.listOfVariables.containsName(observed_value.name):
								t_variable = self.workingModel.listOfVariables.getByName(observed_value.name)
							else:
								raise UnknownTreatmentException(
									"Cannot find a variable called %s" % observed_value.name)

						elif observed_value.name_attribute == "id":
							if self.workingModel.listOfVariables.containsSbmlId(observed_value.name):
								t_variable = self.workingModel.listOfVariables.getBySbmlId(observed_value.name)
							else:
								raise UnknownTreatmentException(
									"Cannot find a variable called %s" % observed_value.name)

						else:
							raise UnknownTreatmentException(
								"Unknown attribute for variable : %s" % observed_value.name_attribute)

						if t_variable is not None and t_variable not in list(vars_observed.keys()):
							vars_observed.update({t_variable: len(list(vars_observed.keys()))})

							nb_observable += 1

					if nb_observable == 0 and self.hasObservations:
						raise NoObservationException("No observation !")

					if nb_treatments == 0 and self.hasTreatments:
						raise NoTreatmentException("No treatments !")

	def writeDataFiles(self):

		f_c = open(self.getTempDirectory() + Settings.C_generatedDirectory_v2 + "data.c", 'w')
		f_h = open(self.getTempDirectory() + Settings.C_generatedDirectory_v2 + "data.h", 'w')

		self.writeDataHeaders(f_c, f_h)
		self.writeDataInitialization(f_c, f_h)
		self.writeDataFinalization(f_c, f_h)

		f_c.close()
		f_h.close()


	def writeDataHeaders(self, f_c, f_h):

		#Writing headers for the c file
		f_c.write("/*****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *   data.c                                                      *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *   written by Vincent Noel                                     *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *   Data definition for optimization                            *\n")
		f_c.write(" *   Generated by libSigNetSim                                      *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" * Copyright (C) 2012-2015 Vincent Noel                          *\n")
		f_c.write(" * the full GPL copyright notice can be found in lsa.c           *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" ****************************************************************/\n\n")
		f_c.write("#include \"data.h\"\n")
		f_c.write("#include <stdlib.h>\n\n")
		f_c.write("Experiment * experiments;\n")
		f_c.write("int nb_experiments;\n\n")
		f_c.write("Experiment * getListOfExperiments()\n")
		f_c.write("{\n")
		f_c.write("  return experiments;\n")
		f_c.write("}\n\n")
		f_c.write("int getNbExperiments()\n")
		f_c.write("{\n")
		f_c.write("  return nb_experiments;\n")
		f_c.write("}\n")

		# Writing headers for the h file
		f_h.write("/*****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *   data.h                                                      *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *   written by Vincent Noel                                     *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *   Data definition for optimization                            *\n")
		f_h.write(" *   Generated by libSigNetSim                                      *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" * Copyright (C) 2012-2015 Vincent Noel                          *\n")
		f_h.write(" * the full GPL copyright notice can be found in lsa.c           *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *****************************************************************/\n\n")

		f_h.write("#include \"integrate/datas.h\"\n\n")


		f_h.write("Experiment * getListOfExperiments();\n")
		f_h.write("int getNbExperiments();\n")


	def writeDataInitialization(self, f_c, f_h):

		f_h.write("void init_data();\n")
		f_c.write("void init_data()\n")
		f_c.write("{\n")

		if self.listOfExperiments is not None and len(self.listOfExperiments) > 0:
			f_c.write("    nb_experiments = %d;\n" % len(self.listOfExperiments))
			f_c.write("    experiments = malloc(sizeof(Experiment)*nb_experiments);\n")
			f_c.write("\n")


			for i, experiment in enumerate(self.listOfExperiments):

				f_c.write("    experiments[%d].name = \"%s\";\n" % (i, experiment.name))
				f_c.write("    experiments[%d].nb_conditions = %d;\n" % (i, len(list(experiment.listOfConditions.keys()))))
				f_c.write("    experiments[%d].conditions = malloc(sizeof(ExperimentalCondition)*experiments[%d].nb_conditions);\n" % (i,i))
				f_c.write("\n")

				for j, condition in enumerate(experiment.listOfConditions.values()):

					# Here we need to find all the timings of the treatments. That will be the set of initial values.
					# Then, each set of initial values can have multiple assignments
					t_times = []
					for initial_value in list(condition.listOfInitialConditions.values()):
						t_times.append(initial_value.t)

					# Removing doublons
					t_times = list(set(t_times))

					# Initial values
					f_c.write("    experiments[%d].conditions[%d].name = \"%s\";\n" % (i, j, condition.name))
					f_c.write("    experiments[%d].conditions[%d].nb_timed_treatments = %d;\n" % (i, j, len(t_times)))
					f_c.write("    experiments[%d].conditions[%d].timed_treatments = malloc(sizeof(TimedTreatments)*experiments[%d].conditions[%d].nb_timed_treatments);\n" % (i,j,i,j))
					f_c.write("\n")

					for k, t_time in enumerate(t_times):

						treatments = []
						for initial_value in list(condition.listOfInitialConditions.values()):
							if initial_value.t == t_time:
								treatments.append(initial_value)

						f_c.write("  experiments[%d].conditions[%d].timed_treatments[%d].t = %g;\n" % (i, j, k, t_time))
						f_c.write("  experiments[%d].conditions[%d].timed_treatments[%d].nb_treatments = %d;\n" % (i, j, k, len(treatments)))
						f_c.write("  experiments[%d].conditions[%d].timed_treatments[%d].treatments = malloc(sizeof(Treatment)*experiments[%d].conditions[%d].timed_treatments[%d].nb_treatments);\n" % (i,j,k,i,j,k))
						for l, treatment in enumerate(treatments):

							t_variable = None
							if self.mapping is not None and len(self.mapping) > 0:
								if i < len(self.mapping) and treatment.name in list(self.mapping[i].keys()):
									# print "> MApping: %s : %s" % (treatment.name, self.mapping[i][treatment.name])
									t_variable = self.workingModel.parentDoc.getByXPath(
										self.mapping[i][treatment.name],
										instance=self.workingModel.parentDoc.useCompPackage
									)
								# else:
								# 	raise UnknownTreatmentException(
								# 		"Variable %s not found in mapping" % treatment.name)

							elif treatment.name_attribute == "name":
								if self.workingModel.listOfVariables.containsName(treatment.name):
									t_variable = self.workingModel.listOfVariables.getByName(treatment.name)
								else:
									raise UnknownTreatmentException("Cannot find a variable called %s" % treatment.name)

							elif treatment.name_attribute == "id":
								if self.workingModel.listOfVariables.containsSbmlId(treatment.name):
									t_variable = self.workingModel.listOfVariables.getBySbmlId(treatment.name)
								else:
									raise UnknownTreatmentException("Cannot find a variable called %s" % treatment.name)

							else:
								raise UnknownTreatmentException("Unknown attribute for variable : %s" % treatment.name_attribute)


							if t_variable is not None:
								f_c.write("  experiments[%d].conditions[%d].timed_treatments[%d].treatments[%d] = (Treatment) {%g, %d, %d};\n" % (
											i, j, k, l, treatment.value, t_variable.type, t_variable.ind))
							# else:
							# 	print "> ERROR: Couldn't find variable %s" % observed_value.name

					 # Observed_values
					f_c.write("    experiments[%d].conditions[%d].nb_observed_values = %d;\n" % (i, j, len(list(condition.listOfExperimentalData.keys()))))
					f_c.write("    experiments[%d].conditions[%d].observed_values = malloc(sizeof(ExperimentalObservation)*experiments[%d].conditions[%d].nb_observed_values);\n" % (i,j,i,j))
					f_c.write("\n")
					vars_observed = {}

					for k, observed_value in enumerate(condition.listOfExperimentalData.values()):

						t_variable = None
						if self.mapping is not None and len(self.mapping) > 0:
							if i < len(self.mapping) and observed_value.name in list(self.mapping[i].keys()):
								# print "> MApping: %s : %s" % (observed_value.name, self.mapping[i][observed_value.name])
								t_variable = self.workingModel.parentDoc.getByXPath(self.mapping[i][observed_value.name], instance=self.workingModel.parentDoc.useCompPackage)
							# else:
							# 	raise UnknownObservationException("Variable %s not found in mapping" % observed_value.name)

						elif observed_value.name_attribute == "name":
							if self.workingModel.listOfVariables.containsName(observed_value.name):
								t_variable = self.workingModel.listOfVariables.getByName(observed_value.name)
							else:
								raise UnknownObservationException("Cannot find a variable called %s" % observed_value.name)

						elif observed_value.name_attribute == "id":
							if self.workingModel.listOfVariables.containsSbmlId(observed_value.name):
								t_variable = self.workingModel.listOfVariables.getBySbmlId(observed_value.name)
							else:
								raise UnknownObservationException("Cannot find a variable called %s" % observed_value.name)

						else:
							raise UnknownObservationException(
								"Unknown attribute for variable : %s" % observed_value.name_attribute)


						if t_variable is not None:
							if t_variable not in list(vars_observed.keys()):
								vars_observed.update({t_variable: len(list(vars_observed.keys())) })

							# if t_variable is not None:
							t_variable_id = t_variable.getPos()

							f_c.write("    experiments[%d].conditions[%d].observed_values[%d] = (ExperimentalObservation) {%g, %g, %g, %d, %g, %g, %d, %d, %d, %d, \"%s\"};\n" % (
											i,j,k,
											observed_value.t, observed_value.value, observed_value.value_dev,
											int(observed_value.steady_state), float(observed_value.min_steady_state), float(observed_value.max_steady_state),
											t_variable.type, t_variable.ind, t_variable_id, vars_observed[t_variable], t_variable.getXPath()))
							# else:
							# 	print "> ERROR: Couldn't find variable %s" % observed_value.name
		else:
			f_c.write("    nb_experiments = 0;\n")
			f_c.write("\n")

		f_c.write("\n}\n\n")



	def writeDataFinalization(self, f_c, f_h):

		f_h.write("void finalize_data();\n")
		f_c.write("void finalize_data()\n")
		f_c.write("{\n")
		if self.listOfExperiments is not None and len(self.listOfExperiments) > 0:
			# print self.listOfExperiments
			for i, experiment in enumerate(self.listOfExperiments):
				for j, condition in enumerate(experiment.listOfConditions.values()):

					if len(list(condition.listOfInitialConditions.keys())) > 0:
						f_c.write("    free(experiments[%d].conditions[%d].timed_treatments);\n" % (i,j))

						t_times = []
						for initial_value in list(condition.listOfInitialConditions.values()):
							t_times.append(initial_value.t)

						# Removing doublons
						t_times = list(set(t_times))

						for k, t_time in enumerate(t_times):
							f_c.write("    free(experiments[%d].conditions[%d].timed_treatments[%d].treatments);\n" % (i,j,k))


					if len(list(condition.listOfExperimentalData.keys())) > 0:
						f_c.write("    free(experiments[%d].conditions[%d].observed_values);\n" % (i,j))

				f_c.write("    free(experiments[%d].conditions);\n" % i)
			f_c.write("    free(experiments);\n")
		f_c.write("\n}\n\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.sedml.HasId import HasId
from libsignetsim.sedml.container.ListOfChanges import ListOfChanges
from libsignetsim.sedml.Source import Source
from libsignetsim.sedml.SedmlException import SedmlModelLanguageNotSupported
from libsignetsim.model.SbmlDocument import SbmlDocument
from libsignetsim.settings.Settings import Settings
from os.path import exists, join, basename
from lxml import etree


class Model(SedBase, HasId):

	LANGUAGE_SBML = "urn:sedml:language:sbml"

	def __init__(self, document):

		SedBase.__init__(self, document)
		HasId.__init__(self, document)

		self.__document = document
		self.__language = None
		self.__source = Source(self.__document)
		self.__sbmlModel = None

		self.listOfChanges = ListOfChanges(self.__document, self)
		self.__nbChanges = 0
		self.__changesApplied = False

	def readSedml(self, model, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, model, level, version)
		HasId.readSedml(self, model, level, version)

		if model.isSetLanguage():
			self.__language = model.getLanguage()

			if not self.__language.startswith(self.LANGUAGE_SBML):

				if len(self.__language.split(":")) == 4:
					raise SedmlModelLanguageNotSupported("Language %s not supported" % self.__language.split(":")[3])
				else:
					raise SedmlModelLanguageNotSupported("Language %s not supported" % self.__language)

		if model.isSetSource():
			self.__source.readSedml(model.getSource(), level, version)

		self.listOfChanges.readSedml(model.getListOfChanges(), level, version)

	def writeSedml(self, model, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, model, level, version)
		HasId.writeSedml(self, model, level, version)

		if self.__language is not None:
			model.setLanguage(self.__language)

		if self.__source.writeSedml(level, version) is not None:
			model.setSource(str(self.__source.writeSedml(level, version)))

		self.listOfChanges.writeSedml(model.getListOfChanges(), level, version)

	def getSbmlModel(self):

		if self.__sbmlModel is None or self.__nbChanges != len(self.listOfChanges):

			self.__loadModel()
			self.__nbChanges = len(self.listOfChanges)

			if len(self.listOfChanges) > 0:
				self.listOfChanges.applyChanges(self.__sbmlModel)
				self.__changesApplied = True
		return self.__sbmlModel

	def __loadModel(self):
		sbml_doc = SbmlDocument()

		if self.listOfChanges.nbXMLChanges() > 0:
			with open(self.__source.getFilename(), 'rb') as sbml_file:
				sbmlTree = etree.XML(sbml_file.read())
				self.listOfChanges.applyXMLChanges(sbmlTree)
				sbml_doc.readSbmlFromString(etree.tostring(sbmlTree).decode('utf-8'))
		else:
			sbml_doc.readSbmlFromFile(self.__source.getFilename())

		self.__sbmlModel = sbml_doc.getModelInstance()

	def makeRelativePath(self, path):
		self.__source.makeRelativePath(path)

	def writeSbmlModelToPath(self, path):

		if self.__sbmlModel is None or self.__changesApplied:
			self.__loadModel()

		self.__sbmlModel.parentDoc.writeSbmlToFile(self.__source.getSource(), path)
		self.__source.setSource(basename(self.__source.getFilename()))

	def setLanguageSbml(self):
		self.__language = self.LANGUAGE_SBML

	def setSource(self, source):
		self.__source.setSource(source)

	def getSource(self):
		return self.__source

	def removePaths(self):
		self.__source.removePaths()

	def makeLocalSource(self):
		return self.__source.makeLocalSource()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasParentObj import HasParentObj

from libsignetsim.model.sbml.Species import Species
from libsignetsim.model.ModelException import CannotDeleteException, InvalidXPath
from libsignetsim.settings.Settings import Settings

from re import match

class ListOfSpecies(ListOf, HasIds, SbmlObject, HasParentObj):

	def __init__ (self, model, parent_obj):

		self.__model = model

		HasParentObj.__init__(self, parent_obj)
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		SbmlObject.__init__(self, model)


	def readSbml(self, sbml_listOfSpecies,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		for sbml_species in sbml_listOfSpecies:
			t_species = Species(self.__model, self, self.nextId())
			t_species.readSbml(sbml_species, sbml_level, sbml_version)
			ListOf.add(self, t_species)

		SbmlObject.readSbml(self, sbml_listOfSpecies, sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		for species in self:
			species.writeSbml(sbml_model, sbml_level, sbml_version)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbml_model.getListOfSpecies(), sbml_level, sbml_version)


	def new(self, name=None, compartment=None, value=0, unit=None,
			 constant=False, boundaryCondition=False, hasOnlySubstanceUnits=False):

		t_species = Species(self.__model, self, self.nextId())
		t_species.new(name, compartment, value, unit, constant,
							boundaryCondition, hasOnlySubstanceUnits)
		ListOf.add(self, t_species)
		return t_species

	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factor=None):

		if obj not in deletions:

			SbmlObject.copy(self, obj)
			for species in obj:

				if species not in deletions:

					t_species = Species(self.__model, self, self.nextId())
					t_species.copy(
						species,
						sids_subs=sids_subs,
						symbols_subs=symbols_subs,
						usids_subs=usids_subs,
						conversion_factor=conversion_factor
					)
					ListOf.add(self, t_species)

	def nbFormulaInitialization(self):

		count = 0
		for species in self:
			if ((species.isConcentration() or species.isDeclaredConcentration)
				and not self.__model.listOfInitialAssignments.hasInitialAssignment(species)):
				count += 1
		return count

	def hasBoundaryConditions(self):

		for species in self:
			if species.boundaryCondition:
				return True

		return False


	def remove(self, species):
		""" Remove an object from the list """

		if species.isInReactions():
			raise CannotDeleteException("Species is used in reactions")
		elif species.isInRules():
			raise CannotDeleteException("Species in used in rules")

		self.__model.listOfVariables.removeVariable(species)
		# self.__model.listOfSbmlIds.removeSbmlId(species)
		ListOf.remove(self, species)


	def removeById(self, species_obj_id):
		""" Remove an object from the list """

		self.remove(self.getById(species_obj_id))

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		for obj in self:
			Species.renameSbmlId(obj, old_sbml_id, new_sbml_id)



	def resolveXPath(self, selector):

		if not (selector.startswith("species") or selector.startswith("sbml:species")):
			raise InvalidXPath(selector)

		res_match = match(r'(.*)\[@(.*)=(.*)\]', selector)
		if res_match is None:
			raise InvalidXPath(selector)

		tokens = res_match.groups()
		if len(tokens) != 3:
			raise InvalidXPath(selector)

		object = None
		if tokens[1] == "id":
			object = self.getBySbmlId(tokens[2][1:-1])
		elif tokens[1] == "name":
			object = self.getByName(tokens[2][1:-1])
		elif tokens[1] == "metaid":
			object = self.getByMetaId(tokens[2][1:-1])

		if object is not None:
			return object

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)

	def getXPath(self):
		return "/".join([self.getParentObj().getXPath(), "sbml:listOfSpecies"])






#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.ModelException import UnknownSIdRefException


class HasConversionFactor(object):
	""" HasConversionFactor property class """


	def __init__ (self, model):
		""" Constructor of class """

		self.__model = model
		self.__conversionFactor = None

	def copy(self, obj):
		self.__conversionFactor = obj.getRawConversionFactor()

	def readSbml(self, conversion_factor,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		self.__conversionFactor = conversion_factor

	def writeSbml(self, object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		if self.__conversionFactor is not None:
			object.setConversionFactor(self.__conversionFactor)

	def isSetConversionFactor(self):
		return self.__conversionFactor is not None

	def getRawConversionFactor(self):
		return self.__conversionFactor

	def getConversionFactor(self):
		if self.__conversionFactor is not None:
			formula = MathFormula(self.__model)
			variable = self.__model.listOfVariables.getBySbmlId(self.__conversionFactor)
			formula.setInternalMathFormula(variable.symbol.getInternalMathFormula())
			return formula

	def getVariableConversionFactor(self):
		if self.__conversionFactor is not None:
			return self.__model.listOfVariables.getBySbmlId(self.__conversionFactor)

	def getSymbolConversionFactor(self):
		if self.__conversionFactor is not None:
			variable = self.__model.listOfVariables.getBySbmlId(self.__conversionFactor)
			return variable.symbol.getInternalMathFormula()

	def setConversionFactor(self, sbml_id):
		if sbml_id is not None and not self.__model.listOfVariables.containsSbmlId(sbml_id):
			raise UnknownSIdRefException("Variable reference %s is unknown" % sbml_id)

		self.__conversionFactor = sbml_id




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.sbml.Rule import Rule
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings
from libsbml import SBML_SPECIES_CONCENTRATION_RULE,\
					SBML_PARAMETER_RULE,\
					SBML_COMPARTMENT_VOLUME_RULE

from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyInteger, SympyMul, SympyPow
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs


class RateRule(Rule):
	""" Class for rate rules """

	def __init__ (self, model, obj_id):

		self.__model = model
		Rule.__init__(self, model, obj_id, Rule.RULE_RATE)

		self.__definition = MathFormula(model, MathFormula.MATH_RATERULE)
		self.__var = None

	def readSbml(self, rate_rule, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		Rule.readSbml(self, rate_rule, sbml_level, sbml_version)

		if self.__model.listOfVariables.containsSbmlId(rate_rule.getVariable()):
			self.__var = self.__model.listOfVariables.getBySbmlId(rate_rule.getVariable())
			self.getVariable().setRuledBy(self)

		self.__definition.readSbml(rate_rule.getMath())

		if self.__definition.getInternalMathFormula() is not None:
			if self.getVariable().isConcentration():
				self.__definition.setInternalMathFormula(
						SympyMul(self.__definition.getInternalMathFormula(),
						self.getVariable().getCompartment().symbol.getInternalMathFormula()))

	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		rate_rule = sbml_model.createRateRule()
		if sbml_level < 2:

			if self.getVariable().isSpecies():
				rate_rule.setL1TypeCode(SBML_SPECIES_CONCENTRATION_RULE)
			elif self.getVariable().isParameter():
				rate_rule.setL1TypeCode(SBML_PARAMETER_RULE)
			elif self.getVariable().isCompartment():
				rate_rule.setL1TypeCode(SBML_COMPARTMENT_VOLUME_RULE)

		Rule.writeSbml(self, rate_rule, sbml_level, sbml_version)

		rate_rule.setVariable(self.__var.getSbmlId())

		if self.__definition.getInternalMathFormula() is not None:
			t_definition = MathFormula(self.__model, MathFormula.MATH_RATERULE)
			t_definition.setInternalMathFormula(self.__definition.getInternalMathFormula())

			if self.getVariable().isConcentration():
				t_definition.setInternalMathFormula(
						SympyMul(t_definition.getInternalMathFormula(),
						SympyPow(self.getVariable().getCompartment().symbol.getInternalMathFormula(),
						SympyInteger(-1))))

			rate_rule.setMath(t_definition.getSbmlMathFormula(sbml_level, sbml_version))

	def copy(self, obj, sids_subs={}, symbols_subs={}, conversion_factors={}, time_conversion=None):

		Rule.copy(self, obj)

		if obj.getVariable().getSbmlId() in list(sids_subs.keys()):
			self.__var = self.__model.listOfVariables.getBySbmlId(sids_subs[obj.getVariable().getSbmlId()])
		else:
			self.__var = self.__model.listOfVariables.getBySbmlId(obj.getVariable().getSbmlId())

		self.getVariable().setRuledBy(self)

		if obj.getDefinition().getInternalMathFormula() is not None:
			t_convs = {}
			for var, conversion in list(conversion_factors.items()):
				t_convs.update({var: var/conversion})

			t_definition = unevaluatedSubs(obj.getDefinition().getInternalMathFormula(), symbols_subs)
			t_definition = unevaluatedSubs(t_definition, t_convs)

			t_var_symbol = unevaluatedSubs(obj.getVariable().symbol.getInternalMathFormula(), symbols_subs)

			if t_var_symbol in conversion_factors:
				t_definition *= conversion_factors[t_var_symbol]

			if time_conversion is not None:
				t_definition /= time_conversion.getInternalMathFormula()

			self.__definition.setInternalMathFormula(t_definition)

	def getVariable(self):
		return self.__var

	def setVariable(self, variable):

		if self.__var is not None:
			self.getVariable().unsetRuledBy()

		self.__var = variable
		self.getVariable().setRuledBy(self)

	def setPrettyPrintDefinition(self, definition, rawFormula=False):

		if self.getVariable().isConcentration() and not rawFormula:
			t_comp = self.getVariable().getCompartment()
			t_math_formula = MathFormula(self.__model, MathFormula.MATH_RATERULE)
			t_math_formula.setPrettyPrintMathFormula(definition, rawFormula=rawFormula)
			self.__definition.setInternalMathFormula(
				t_math_formula.getInternalMathFormula()
				* t_comp.symbol.getInternalMathFormula())

		else:
			self.__definition.setPrettyPrintMathFormula(definition, rawFormula=rawFormula)

	def getPrettyPrintDefinition(self):

		if self.__definition.getInternalMathFormula() is not None:
			if self.getVariable().isConcentration():
				t_comp = self.getVariable().getCompartment()
				t_math_formula = MathFormula(self.__model, MathFormula.MATH_ASSIGNMENTRULE)
				t_math_formula.setInternalMathFormula(
					self.__definition.getInternalMathFormula()/t_comp.symbol.getInternalMathFormula()
				)
				return t_math_formula.getPrettyPrintMathFormula()

			else:
				return self.__definition.getPrettyPrintMathFormula()

	def getRawDefinition(self, rawFormula=False):

		formula = self.__definition.getInternalMathFormula()

		if formula is not None:
			if self.getVariable().isConcentration() and not rawFormula:
				formula /= self.getVariable().getCompartment().symbol.getInternalMathFormula()

			if not rawFormula:
				subs = {}
				for species in self.__model.listOfSpecies:
					if species.isConcentration():
						subs.update({species.symbol.getInternalMathFormula(rawFormula=True): species.symbol.getInternalMathFormula()})
				formula = unevaluatedSubs(formula, subs)
		return formula

	def getDefinition(self, rawFormula=False):

		t_formula = MathFormula(self.__model, MathFormula.MATH_RATERULE)
		t_formula.setInternalMathFormula(self.getRawDefinition(rawFormula=rawFormula))
		return t_formula

	def setDefinition(self, definition):
		self.__definition = definition

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		self.__definition.renameSbmlId(old_sbml_id, new_sbml_id)

	def containsVariable(self, variable):
		if self.__definition.getInternalMathFormula() is not None:
			return (variable.symbol.getInternalMathFormula() in self.__definition.getInternalMathFormula().atoms()
					or (variable.isSpecies() and SympySymbol("_speciesForcedConcentration_%s_" % str(variable.symbol.getInternalMathFormula())) in self.__definition.getInternalMathFormula().atoms())
					or variable.symbol.getInternalMathFormula() == self.getVariable().symbol.getInternalMathFormula())
		else:
			return False

	def isValid(self):
		return (
			self.getVariable() is not None
			and self.__definition.getInternalMathFormula() is not None
			and self.__definition.getDeveloppedInternalMathFormula() is not None
		)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject

from libsignetsim.model.sbml.Port import Port
from libsignetsim.settings.Settings import Settings

class ListOfPorts(ListOf, HasIds):#, SbmlObject):
    """ Class for the listOfModelDefinition in a sbml model """

    def __init__ (self, model=None):

        self.__model = model
        ListOf.__init__(self, model)
        HasIds.__init__(self, model)
        # SbmlObject.__init__(self, model)


    def readSbml(self, sbml_list_ports,
                    sbml_level=Settings.defaultSbmlLevel,
                    sbml_version=Settings.defaultSbmlVersion):
        """ Reads compartments' list from a sbml file """

        for port in sbml_list_ports:
            t_port = Port(self.__model, self.nextId())
            t_port.readSbml(port, sbml_level, sbml_version)
            ListOf.add(self, t_port)

        # SbmlObject.readSbml(self, sbml_list_ports, sbml_level, sbml_version)


    def writeSbml(self, sbml_model,
                    sbml_level=Settings.defaultSbmlLevel,
                    sbml_version=Settings.defaultSbmlVersion):
        """ Writes compartments' list to a sbml file """

        for t_port in self:
            sbml_port = sbml_model.createPort()
            t_port.writeSbml(sbml_port, sbml_level, sbml_version)

        # SbmlObject.writeSbml(self, sbml_model, sbml_level, sbml_version)


    def new(self):
        """ Creates a new compartment """

        t_port = Port(self.__model, self.nextId())
        # t_deletion.new(name, sbml_id)
        ListOf.add(self, t_port)
        return t_port


    def remove(self, comp):
        """ Remove an object from the list """

        ListOf.remove(self, comp)


    def removeById(self, obj_id):
        """ Remove an object from the list """
        self.remove(self.getById(obj_id))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings

class SbmlModelCreator(object):
	""" Sbml model creator class """


	def __init__ (self):
		""" Constructor of model class """

		self.__familyName = None
		self.__givenName = None
		self.__email = None
		self.__organisation = None
		self.__name = None

	def new(self, family_name, given_name, email, organisation=None, name=None):

		self.__familyName = family_name
		self.__givenName = given_name
		self.__email = email
		self.__organisation = organisation
		self.__name = name

	def readSbml(self, creator, level=Settings.defaultSbmlLevel, version=Settings.defaultSbmlVersion):

		if creator.isSetFamilyName():
			self.__familyName = creator.getFamilyName()

		if creator.isSetGivenName():
			self.__givenName = creator.getGivenName()

		if creator.isSetEmail():
			self.__email = creator.getEmail()

		if creator.isSetOrganisation():
			self.__organisation = creator.getOrganisation()

	def writeSbml(self, creator, level=Settings.defaultSbmlLevel, version=Settings.defaultSbmlVersion):

		if self.__familyName is not None:
			creator.setFamilyName(str(self.__familyName))

		if self.__givenName is not None:
			creator.setGivenName(str(self.__givenName))

		if self.__email is not None:
			creator.setEmail(str(self.__email))

		if self.__organisation is not None:
			creator.setOrganisation(str(self.__organisation))

	def getFamilyName(self):
		return self.__familyName

	def getGivenName(self):
		return self.__givenName

	def getEmail(self):
		return self.__email

	def getOrganisation(self):
		return self.__organisation

	def getName(self):
		return self.__name

	def setFamilyName(self, family_name):
		self.__familyName = family_name

	def setGivenName(self, given_name):
		self.__givenName = given_name

	def setEmail(self, email):
		self.__email = email

	def setOrganization(self, organisation):
		self.__organisation = organisation


	def __str__(self):

		return "%s %s (%s)" % (self.__givenName, self.__familyName, self.__email)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.NMBase import NMBase
from libsignetsim.settings.Settings import Settings

class DimensionDescription (NMBase):

	def __init__(self, document, name=None):

		NMBase.__init__(self, document)
		self.__document = document
		self.__id = None
		self.__name = name
		# self.__ontologyTerm = None

	def readNuML(self, dimension_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		NMBase.readNuML(self, dimension_description, level, version)

		self.__id = dimension_description.getId()
		if dimension_description.isSetName():
			self.__name = dimension_description.getName()

		# self.__ontologyTerm = self.__document.listOfOntologyTerms.getByOntologyTerm(dimension_description.getOntologyTerm())


	def writeNuML(self, dimension_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		NMBase.writeNuML(self, dimension_description, level, version)

		if self.__id is not None:
			dimension_description.setId(self.__id)

		if self.__name is not None:
			dimension_description.setName(self.__name)

		# if self.__ontologyTerm is not None:
		# 	dimension_description.setOntologyTerm(self.__ontologyTerm.getId())

	def getName(self):
		return self.__name

	def getId(self):
		return self.__id

	def setId(self, description_id):
		self.__id = description_id


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject

from libsignetsim.model.sbml.Deletion import Deletion
from libsignetsim.settings.Settings import Settings

class ListOfDeletions(ListOf, HasIds):#, SbmlObject):
	""" Class for the listOfModelDefinition in a sbml model """

	def __init__ (self, model, parent_submodel):

		self.__model = model
		self.__parentSubmodel = parent_submodel
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		# SbmlObject.__init__(self, model)

	def readSbml(self, sbml_list_deletions,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads compartments' list from a sbml file """

		for deletion in sbml_list_deletions:
			t_deletion = Deletion(self.__model, self.nextId(), self.__parentSubmodel)
			t_deletion.readSbml(deletion, sbml_level, sbml_version)
			ListOf.add(self, t_deletion)

		# SbmlObject.readSbml(self, sbml_list_deletions, sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes compartments' list to a sbml file """

		for t_deletion in self:
			sbml_deletion = sbml_model.createDeletion()
			t_deletion.writeSbml(sbml_deletion, sbml_level, sbml_version)

		# SbmlObject.writeSbml(sbml_model)


	def new(self):
		""" Creates a new compartment """

		t_deletion = Deletion(self.__model, self.nextId(), self.__parentSubmodel)
		ListOf.add(self, t_deletion)
		return t_deletion


	def remove(self, deletion):
		""" Remove an object from the list """
		ListOf.remove(self, deletion)

	def removeById(self, obj_id):
		""" Remove an object from the list """
		self.remove(self.getById(obj_id))


	def getDeletedMetaIds(self):

		objs = []
		for deletion in self:
			objs.append(deletion.getRefObject().getMetaId())

		return objs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from sympy import simplify, expand, srepr
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs
from libsignetsim.model.math.sympy_shortcuts import (
	SympySymbol, SympyInteger, SympyFloat, SympyRational, SympyAtom,
	SympyOne, SympyNegOne, SympyZero, SympyPi, SympyE, SympyExp1, SympyHalf,
	SympyInf, SympyNan, SympyAdd, SympyMul, SympyPow,
	SympyFunction, SympyUndefinedFunction, SympyLambda, SympyDerivative,
	SympyCeiling, SympyFloor, SympyAbs, SympyLog, SympyExp, SympyPiecewise,
	SympyFactorial, SympyRoot, SympyAcos, SympyAsin, SympyAtan, SympyAcosh,
	SympyAsinh, SympyAtanh, SympyCos, SympySin, SympyTan, SympyAcot,
	SympyAcoth, SympyCosh, SympySinh, SympyTanh, SympySec, SympyCsc,
	SympyCot, SympyCoth, SympyAcsc, SympyAsec,
	SympyEqual, SympyUnequal, SympyGreaterThan, SympyLessThan,
	SympyStrictGreaterThan, SympyStrictLessThan,
	SympyAnd, SympyOr, SympyXor, SympyNot, SympyTrue, SympyFalse,
	SympyMax, SympyMin)


class KineticLawIdentifier(object):
	""" Class for handling math formulaes """

	MASS_ACTION = 0
	MICHAELIS = 1
	UNDEFINED = 2
	HILL = 3

	reactionTypes = {MASS_ACTION: "Mass Action reaction",
					MICHAELIS: "Enzymatic reaction",
					UNDEFINED: "User defined reaction",
					HILL: "Hill kinetics reaction"}

	#parametersList = {(MASS_ACTION, False): ["Forward kinetic parameter"],
	#                  (MASS_ACTION, True): ["Forward kinetic parameter", "Backward kinetic parameter"],
	#                  (MICHAELIS, False): ["Catalytic constant", "Michaelis constant"],
	#                  }

	allowReversibleList = {

		MASS_ACTION: True,
		MICHAELIS: False,
		UNDEFINED: True,
		HILL: False,
	}

	parametersList = {

		MASS_ACTION: {
			False: ["Kinetic parameter"],
			True: ["Forward kinetic parameter", "Backward kinetic parameter"],
		},

		MICHAELIS: {
			False: ["Catalytic sconstant", "Michaelis constant"],
		},

		UNDEFINED: {
			True: [],
			False: [],
		},

		HILL: {
			False: ["kcat", "Kd", "n"],
		},
	}


	T_PARAM = SympySymbol("_parameter_")
	T_SPECIES = SympySymbol("_species_")
	T_COMPARTMENT = SympySymbol("_compartment_")
	T_REACTANT = SympySymbol("_reactant_")
	T_MODIFIER = SympySymbol("_modifier_")
	T_PRODUCT = SympySymbol("_product_")

	T_MM_1 = SympyMul(SympyMul(SympySymbol("_parameter_"), SympySymbol("_species_")),
						SympyPow(SympyAdd(SympySymbol("_parameter_"), SympySymbol("_species_")),
									SympyInteger(-1)))
	T_MM_2 = SympyMul(SympyMul(SympySymbol("_parameter_"), SympyPow(SympySymbol("_species_"), SympyInteger(2))),
						SympyPow(SympyAdd(SympySymbol("_parameter_"), SympySymbol("_species_")),
									SympyInteger(-1)))


	T_MA_IR = SympyMul(SympySymbol("_parameter_"), SympySymbol("_reactant_"))
	T_MA_R = SympyMul(SympySymbol("_parameter_"), SympySymbol("_reactant_")) - SympyMul(SympySymbol("_parameter_"), SympySymbol("_product_"))


	T_MM_WITHOUT_ENZYME = SympyMul(SympyMul(SympySymbol("_parameter_"), SympySymbol("_reactant_")),
						SympyPow(SympyAdd(SympySymbol("_parameter_"), SympySymbol("_reactant_")),
									SympyInteger(-1)))

	T_MM_WITH_ENZYME = SympyMul(SympyMul(SympySymbol("_parameter_"), SympySymbol("_reactant_"), SympySymbol("_modifier_")),
						SympyPow(SympyAdd(SympySymbol("_parameter_"), SympySymbol("_reactant_")),
									SympyInteger(-1)))

	ZERO = SympyInteger(0)

	def __init__(self, model, reaction):
		""" Constructor """

		self.model = model
		self.reaction = reaction

		self.reactionType = None

		self.reversible = None
		self.forwardMathFormula = None
		self.reactionTypeForward = None
		self.backwardMathFormula = None
		self.reactionTypeBackward = None

		self.mathRate = None
		self.typeRate = None

	def removeCompartmentsFromRate(self, formula=None):

		if formula is None:
			t_rate = self.getDefinition().getDeveloppedInternalMathFormula()
		else:
			t_rate = formula

		t_replaces = {}

		for compartment in self.model.listOfCompartments:
			t_replaces.update({compartment.symbol.getInternalMathFormula(): SympyInteger(1)})

		return unevaluatedSubs(t_rate, t_replaces)


	def removeSpeciesAndCompartmentsFromRate(self, formula=None):

		if formula is None:
			t_rate = self.getDefinition().getDeveloppedInternalMathFormula()
		else:
			t_rate = formula

		t_replaces = {}

		for compartment in self.model.listOfCompartments:
			t_replaces.update({compartment.symbol.getInternalMathFormula(): SympyInteger(1)})

		for species in self.model.listOfSpecies:
			t_replaces.update({species.symbol.getInternalMathFormula(): SympyInteger(1)})

		return expand(simplify(unevaluatedSubs(t_rate, t_replaces)))


	def simplifyRate(self, formula=None):

		if formula is None:
			t_rate = self.removeCompartmentsFromRate()
		else:
			t_rate = formula

		t_replaces = {}

		for var in self.model.listOfVariables:
			if var.isParameter():
				t_replaces.update({var.symbol.getInternalMathFormula(): self.T_PARAM})
			elif var.isCompartment():
				t_replaces.update({var.symbol.getInternalMathFormula(): SympyInteger(1)})

		for i, reactant in enumerate(self.reaction.listOfReactants):
			if i == 0:
				t_replaces.update({reactant.getSpecies().symbol.getInternalMathFormula(): self.T_REACTANT})
			else:
				t_replaces.update({reactant.getSpecies().symbol.getInternalMathFormula(): SympyInteger(1)})

		for i, modifier in enumerate(self.reaction.listOfModifiers):
			if i == 0:
				t_replaces.update({modifier.getSpecies().symbol.getInternalMathFormula(): self.T_MODIFIER})
			else:
				t_replaces.update({modifier.getSpecies().symbol.getInternalMathFormula(): SympyInteger(1)})

		for i, product in enumerate(self.reaction.listOfProducts):
			if i == 0:
				t_replaces.update({product.getSpecies().symbol.getInternalMathFormula(): self.T_PRODUCT})
			else:
				t_replaces.update({product.getSpecies().symbol.getInternalMathFormula(): SympyInteger(1)})

		return simplify(unevaluatedSubs(t_rate, t_replaces))

	def isReversible(self, formula=None):

		if formula is None:
			formula = self.simplifyRate()

		formula = expand(simplify(formula))
		# If we had an addition
		if formula.func == SympyAdd:

			# And one of the terms
			for arg in formula.args:

				# is *(-1)
				if (arg.func == SympyMul
					and (arg.args[0] == SympyInteger(-1)
						or arg.args[1] == SympyInteger(-1))):

					return True

		return False

	def getReversibleRates(self, formula):

		t_backward = None
		formula = expand(simplify(formula))
		# If we had an addition
		if formula.func == SympyAdd:
			# And one of the terms
			for arg in formula.args:
				# is *(-1)
				if arg.func == SympyMul:
					if (arg.args[0] == SympyInteger(-1)
						or arg.args[1] == SympyInteger(-1)):
						t_backward = arg*SympyInteger(-1)
		else:
			return (formula,SympyInteger(0))

		t_forward = SympyAdd(formula, t_backward)

		return (t_forward, t_backward)


	def findKineticLaw(self):

		# print "> Looking for the kinetic law"

		self.reactionType = self.UNDEFINED

		self.mathRate = self.removeCompartmentsFromRate()
		self.typeRate = self.simplifyRate()
		self.reversible = False

		t_formula = expand(simplify(self.typeRate))

		if simplify(self.typeRate - self.T_MA_IR) == 0 or simplify(self.typeRate - self.T_MA_R) == 0:
			self.reactionType = self.MASS_ACTION

			if self.isReversible(t_formula):
				self.reversible = True
				self.reaction.reversible = True
				self.getReversibleFormulas()
		elif (
			simplify(self.typeRate - self.T_MM_WITH_ENZYME) == 0
			or simplify(self.typeRate - self.T_MM_WITHOUT_ENZYME) == 0
		):
			self.reactionType = self.MICHAELIS

		else:
			self.reactionType = self.UNDEFINED

	def getReversibleFormulas(self):

		t_formula = self.getDefinition().getDeveloppedInternalMathFormula()
		for compartment in self.model.listOfCompartments:
			t_formula = t_formula.subs(compartment.symbol.getInternalMathFormula(), SympyInteger(1))

		t_formula = expand(simplify(t_formula))
		found = False

		if t_formula.func == SympyAdd:

			if t_formula.args[0].func == SympyMul:
				if (t_formula.args[0].args[0] == SympyInteger(-1)
					or t_formula.args[0].args[1] == SympyInteger(-1)):

					self.forwardMathFormula = t_formula.args[1]
					self.backwardMathFormula = t_formula.args[0]*SympyInteger(-1)
					found = True

			if t_formula.args[1].func == SympyMul and not found:
				if (t_formula.args[1].args[0] == SympyInteger(-1)
					or t_formula.args[1].args[1] == SympyInteger(-1)):

					self.forwardMathFormula = t_formula.args[0]
					self.backwardMathFormula = t_formula.args[1]*SympyInteger(-1)

	def getParameters(self):

		if self.reactionType == self.MASS_ACTION and not self.reversible:
			return self.findMassActionParameters()

		elif self.reactionType == self.MASS_ACTION and self.reversible:
			return self.findReversibleMassActionParameters()

		elif self.reactionType == self.MICHAELIS:
			return self.findMichaelisMentenParameters()

	def findMichaelisMentenParameters(self):

		param_catalytic = None
		param_michaelis = None

		t_formula = self.removeSpeciesAndCompartmentsFromRate()

		if t_formula.func == SympyMul:
			if t_formula.args[0].func == SympyPow:

				param_catalytic = t_formula.args[1]

				if t_formula.args[0].args[0].func == SympyAdd:
					if t_formula.args[0].args[0].args[1] == SympyInteger(1):
						param_michaelis = t_formula.args[0].args[0].args[0]
					elif t_formula.args[0].args[0].args[0] == SympyInteger(1):
						param_michaelis = t_formula.args[0].args[0].args[1]

				elif t_formula.args[0].args[1].func == SympyAdd:
					if t_formula.args[0].args[1].args[1] == SympyInteger(1):
						param_michaelis = t_formula.args[0].args[1].args[1]
					elif t_formula.args[0].args[1].args[0] == SympyInteger(1):
						param_michaelis = t_formula.args[0].args[1].args[0]

			elif t_formula.args[1].func == SympyPow:

				param_catalytic = t_formula.args[0]

				if t_formula.args[1].args[0].func == SympyAdd:
					if t_formula.args[1].args[0].args[1] == SympyInteger(1):
						param_michaelis = t_formula.args[1].args[0].args[0]
					elif t_formula.args[1].args[0].args[0] == SympyInteger(1):
						param_michaelis = t_formula.args[1].args[0].args[1]

				elif t_formula.args[1].args[1].func == SympyAdd:
					if t_formula.args[1].args[1].args[0] == SympyInteger(1):
						param_michaelis = t_formula.args[1].args[1].args[1]
					elif t_formula.args[1].args[1].args[1] == SympyInteger(1):
						param_michaelis = t_formula.args[1].args[1].args[0]

		param_catalytic = self.model.listOfVariables.getBySymbol(param_catalytic)
		param_michaelis = self.model.listOfVariables.getBySymbol(param_michaelis)

		return [param_catalytic, param_michaelis]

	def findMassActionParameters(self):

		param = self.removeSpeciesAndCompartmentsFromRate()
		return [self.model.listOfVariables.getBySymbol(param)]

	def findReversibleMassActionParameters(self):

		self.getReversibleFormulas()

		front_param = self.removeSpeciesAndCompartmentsFromRate(self.forwardMathFormula)
		front_param = self.model.listOfVariables.getBySymbol(front_param)

		back_param = self.removeSpeciesAndCompartmentsFromRate(self.backwardMathFormula)
		back_param = self.model.listOfVariables.getBySymbol(back_param)

		return [front_param, back_param]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from sympy.core.symbol import Symbol as SympySymbol
from sympy.core.numbers import Integer as SympyInteger
from sympy.core.numbers import Float as SympyFloat
from sympy.core.basic import Atom as SympyAtom
#numbers
from sympy.core.numbers import One as SympyOne
from sympy.core.numbers import NegativeOne as SympyNegOne
from sympy.core.numbers import Zero as SympyZero
from sympy.core.numbers import pi as SympyPi
from sympy.core.numbers import E as SympyE
from sympy.core.numbers import Exp1 as SympyExp1
from sympy.core.numbers import Half as SympyHalf
from sympy.core.numbers import Rational as SympyRational
from sympy.core.numbers import oo as SympyInf
from sympy.core.numbers import nan as SympyNan

#basic operators
from sympy.core.add import Add as SympyAdd
from sympy.core.mul import Mul as SympyMul
from sympy.core.power import Pow as SympyPow

#advanced operators
from sympy.core.function import Function as SympyFunction
from sympy.core.function import UndefinedFunction as SympyUndefinedFunction
from sympy.core.function import Lambda as SympyLambda
from sympy.core.function import Derivative as SympyDerivative
from sympy.functions.elementary.integers import ceiling as SympyCeiling
from sympy.functions.elementary.integers import floor as SympyFloor
from sympy.functions.elementary.complexes import Abs as SympyAbs
from sympy.functions.elementary.exponential import log as SympyLog
from sympy.functions.elementary.exponential import exp as SympyExp
from sympy.functions.elementary.piecewise import Piecewise as SympyPiecewise
from sympy.functions.elementary.piecewise import ExprCondPair as SympyExprCondPair

from sympy.functions.combinatorial.factorials import factorial as SympyFactorial
from sympy.functions import root as SympyRoot

#trigo
from sympy.functions import acos as SympyAcos
from sympy.functions import asin as SympyAsin
from sympy.functions import atan as SympyAtan
from sympy.functions import acosh as SympyAcosh
from sympy.functions import asinh as SympyAsinh
from sympy.functions import atanh as SympyAtanh
from sympy.functions import cos as SympyCos
from sympy.functions import sin as SympySin
from sympy.functions import tan as SympyTan
from sympy.functions import acot as SympyAcot
from sympy.functions import acoth as SympyAcoth
from sympy.functions import cosh as SympyCosh
from sympy.functions import sinh as SympySinh
from sympy.functions import tanh as SympyTanh
from sympy.functions import sec as SympySec
from sympy.functions import csc as SympyCsc
from sympy.functions import cot as SympyCot
from sympy.functions import coth as SympyCoth
from sympy.functions import acsc as SympyAcsc
from sympy.functions import asec as SympyAsec
# from sympy.mpmath import acsch as SympyAcsch
# from sympy.mpmath import asech as SympyAsech


#comparaison
from sympy.core import Equality as SympyEqual
from sympy.core import Unequality as SympyUnequal
from sympy.core import GreaterThan as SympyGreaterThan
from sympy.core import LessThan as SympyLessThan
from sympy.core import StrictGreaterThan as SympyStrictGreaterThan
from sympy.core import StrictLessThan as SympyStrictLessThan

#logic
from sympy.logic.boolalg import And as SympyAnd
from sympy.logic.boolalg import Or as SympyOr
from sympy.logic.boolalg import Xor as SympyXor
from sympy.logic.boolalg import Not as SympyNot
from sympy.logic.boolalg import BooleanTrue as SympyTrue
from sympy.logic.boolalg import BooleanFalse as SympyFalse
from sympy.logic.boolalg import ITE as SympyITE



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.Range import Range
from libsignetsim.settings.Settings import Settings

from numpy import linspace, logspace, log10
class UniformRange(Range):

	LINEAR = "linear"
	LOG = "log"

	def __init__(self, document):

		Range.__init__(self, document)
		self.__document = document

		self.__start = None
		self.__end = None
		self.__numberOfPoints = None
		self.__type = None

	def readSedml(self, uniform_range, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		Range.readSedml(self, uniform_range, level, version)

		if uniform_range.isSetStart():
			self.__start = uniform_range.getStart()

		if uniform_range.isSetEnd():
			self.__end = uniform_range.getEnd()

		if uniform_range.isSetNumberOfPoints():
			self.__numberOfPoints = uniform_range.getNumberOfPoints()

		if uniform_range.isSetType():
			self.__type = uniform_range.getType()

	def writeSedml(self, uniform_range, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		Range.writeSedml(self, uniform_range, level, version)

		if self.__start is not None:
			uniform_range.setStart(self.__start)

		if self.__end is not None:
			uniform_range.setEnd(self.__end)

		if self.__numberOfPoints is not None:
			uniform_range.setNumberOfPoints(self.__numberOfPoints)

		if self.__type is not None:
			uniform_range.setType(self.__type)

	def getStart(self):
		return self.__start

	def getEnd(self):
		return self.__end

	def getNumberOfPoints(self):
		return self.__numberOfPoints

	def getType(self):
		return self.__type

	def setStart(self, start):
		self.__start = start

	def setEnd(self, end):
		self.__end = end

	def setNumberOfPoints(self, number_of_points):
		self.__numberOfPoints = number_of_points

	def setStype(self, type):
		self.__type = type

	def setLinear(self):
		self.__type = self.LINEAR

	def setLog(self):
		self.__type = self.LOG

	def getValuesArray(self):
		if self.__type == self.LOG:
			return [0]+list(logspace(log10(self.__start), log10(self.__end), self.__numberOfPoints+1))
		else:
			return linspace(self.__start, self.__end, self.__numberOfPoints+1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""


"""

from libsignetsim.model.sbml.HasId import HasId
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.settings.Settings import Settings
from re import match


class ExternalModelDefinition(HasId, SbmlObject, HasParentObj):

	def __init__(self, model, obj_id, parent_obj):

		self.__model = model

		self.objId = obj_id
		HasParentObj.__init__(self, parent_obj)
		HasId.__init__(self, model)
		SbmlObject.__init__(self, model)
		self.__modelRef = None

		self.__source = None
		self.modelDefinition = None

	def readSbml(self, sbml_model_definition,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		HasId.readSbml(self, sbml_model_definition, sbml_level, sbml_version)
		SbmlObject.readSbml(self, sbml_model_definition, sbml_level, sbml_version)

		if sbml_model_definition.isSetModelRef():
			self.__modelRef = sbml_model_definition.getModelRef()

		if sbml_model_definition.isSetSource():
			self.__source = sbml_model_definition.getSource()

		t_id_dep = self.__model.parentDoc.documentDependenciesPaths.index(self.__source)
		t_document = self.__model.parentDoc.documentDependencies[t_id_dep]
		t_document.setParentObj(self)

		if self.__modelRef is not None:
			self.modelDefinition = t_document.getSubmodel(self.__modelRef)
		else:
			self.modelDefinition = t_document.model

	def writeSbml(self, sbml_model_definition,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		HasId.writeSbml(self, sbml_model_definition, sbml_level, sbml_version)

		if self.__modelRef is not None:
			sbml_model_definition.setModelRef(self.__modelRef)

		if self.__source is not None:
			sbml_model_definition.setSource(self.__source)

		SbmlObject.writeSbml(self, sbml_model_definition, sbml_level, sbml_version)

	def getSource(self):
		return self.__source

	def setSource(self, source):
		self.__source = source
		self.updateModelDefinition()

	def hasModelRef(self):
		return self.__modelRef is not None

	def getModelRef(self):
		return self.__modelRef

	def setModelRef(self, model_ref):
		self.__modelRef = model_ref
		self.updateModelDefinition()

	def updateModelDefinition(self):

		self.__model.parentDoc.documentDependenciesPaths = None
		self.__model.parentDoc.loadExternalDocumentDependencies()

		t_id_dep = self.__model.parentDoc.documentDependenciesPaths.index(self.__source)
		t_document = self.__model.parentDoc.documentDependencies[t_id_dep]

		if self.__modelRef is not None:
			self.modelDefinition = t_document.getSubmodel(self.__modelRef)
		else:
			self.modelDefinition = t_document.model

	# def resolveXPath(self, selector):
	#
	# 	if not (selector.startswith("externalModelDefinition") or selector.startswith("sbml:externalModelDefinition")):
	# 		raise InvalidXPath(selector)
	#
	# 	if "[@" in selector:
	# 		res_match = match(r'(.*)\[@(.*)=(.*)\]', selector)
	# 		if res_match is None:
	# 			raise InvalidXPath(selector)
	#
	# 		tokens = res_match.groups()
	# 		if len(tokens) != 3:
	# 			raise InvalidXPath(selector)
	#
	# 		object = None
	# 		if tokens[1] == "id":
	# 			object = self.getBySbmlId(tokens[2][1:-1])
	# 		elif tokens[1] == "name":
	# 			object = self.getByName(tokens[2][1:-1])
	#
	# 		if object is not None:
	# 			return object
	# 	else:
	#
	#
	# 	# If not returned yet
	# 	raise InvalidXPath(selector)
	#
	def getByXPath(self, xpath):
		return self.modelDefinition.parentDoc.getByXPath("/".join(xpath))

	def setByXPath(self, xpath, object):
		self.modelDefinition.parentDoc.setByXPath("/".join(xpath), object)

	def getXPath(self, attribute=None):

		xpath = "sbml:externalModelDefinition"
		if self.__model.sbmlLevel == 1:
			xpath += "[@name='%s']" % self.getSbmlId()
		else:
			xpath += "[@id='%s']" % self.getSbmlId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.ComputeChange import ComputeChange
from libsignetsim.sedml.SedmlException import SedmlUnknownXPATH
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.settings.Settings import Settings


class SetValue(ComputeChange):

	TARGET = 0
	SYMBOL = 1

	def __init__(self, document, repeated_task):

		ComputeChange.__init__(self, document)
		self.__document = document
		self.__repeatedTask = repeated_task
		self.__modelReference = None
		self.__range = None
		self.__variableType = None
		self.__symbol = None

	def readSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		ComputeChange.readSedml(self, change, level, version)

		if change.isSetModelReference():
			self.__modelReference = change.getModelReference()

		if change.isSetRange():
			self.__range = change.getRange()

		if change.isSetSymbol():
			self.__symbol = change.getSymbol()

	def writeSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		ComputeChange.writeSedml(self, change, level, version)

		if self.__modelReference is not None:
			change.setModelReference(self.__modelReference)

		if self.__range is not None:
			change.setRange(self.__range)

		if self.__symbol is not None:
			change.setSymbol(self.__symbol)

	def isTargetChange(self):
		return self.getTarget().getXPath() is not None

	def isSymbolChange(self):
		return self.__symbol is not None

	def setModelReference(self, model_reference):
		self.__modelReference = model_reference

	def setModel(self, model):
		self.__modelReference = model.getId()

	def setRangeReference(self, range_reference):
		self.__range = range_reference

	def setRange(self, range_obj):
		self.__range = range_obj.getId()


	def getValueChange(self):

		if self.isTargetChange():
			try:
				model = self.__document.listOfModels.getSbmlModelByReference(self.__modelReference)
				target = self.getTarget().getModelObject(model)

				range = self.__repeatedTask.listOfRanges.getByRangeId(self.__range)
				array_values = range.getValuesArray()

				return {target: array_values}
			except InvalidXPath as e:
				raise SedmlUnknownXPATH("Unknown XPath : %s" % e.message)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function



from sympy import eye, Matrix, pretty, srepr


class MathConservationMatrix(object):
	""" Sbml model class """

	def __init__(self, model):
		""" Constructor of model class """

		self.__model = model
		self.conservationMatrix = None

	def getConservationMatrix(self, stoichiometry_matrix=None):

		if stoichiometry_matrix is not None:
			return self.buildConservationMatrix(stoichiometry_matrix)

		elif self.conservationMatrix is None:
			self.conservationMatrix = self.buildConservationMatrix(stoichiometry_matrix)

		return self.conservationMatrix

	def buildConservationMatrix(self, stoichiometry_matrix=None):

		conservationMatrix = None
		if stoichiometry_matrix is None:
			sm = self.__model.stoichiometryMatrix.getStoichiometryMatrix()
		else:
			sm = stoichiometry_matrix.getStoichiometryMatrix()

		if sm is not None:
			sm = sm.evalf()
			T0 = sm.row_join(eye(sm.shape[0]))
			n = sm.shape[1]

			Ts = [T0]
			j = 0

			all_zero = False
			while not all_zero:
				Ts.append(self.__next_tableau(Ts[j], j, n))
				j += 1
				all_zero = True
				for i in range(Ts[j].shape[0]):
					all_zero &= all([j_i == 0 for j_i in Ts[j][i, 0:n]])

			last_T = Ts[len(Ts) - 1]
			conservationMatrix = last_T[:, n:n + sm.shape[0]]

		return conservationMatrix
	def __str__(self):
		if self.conservationMatrix is not None:
			return srepr(self.conservationMatrix)

	def pprint(self):
		if self.conservationMatrix is not None:
			print("> Conservation matrix: ")
			print(pretty(self.conservationMatrix))
			print("\n")


	def __buildS(self, T, n):
		S = []
		for i in range(T.shape[0]):
			S.append(set([ii for ii, ii_val in enumerate(T[i, n:T.shape[1] + 1]) if ii_val == 0]))
		return S

	def __getCondition1(self, T, i, j, k):
		return T[i, j] * T[k, j] < 0

	def __getCondition2(self, T, S, i, k):

		intersection = S[i].intersection(S[k])
		result = True
		for l in range(T.shape[0]):
			if l != k and l != i:
				if intersection.issubset(S[l]):
					result = False

		return result

	def __buildIndices(self, T, S, j):

		indices = []
		for i in range(T.shape[0]):
			for k in range(T.shape[0]):
				if i != k and i < k:
					if self.__getCondition1(T, i, j, k) and self.__getCondition2(T, S, i, k):
						indices.append((i, k))

		return indices

	def __buildVectors(self, T, j, indices):

		result = []
		for i, k in indices:
			vector = abs(T[i, j]) * T[k, :] + abs(T[k, j]) * T[i, :]
			result.append(vector)

		return result

	def __buildTp1(self, T, j, vectors):

		T_p1 = Matrix([[]])
		if len(vectors) > 0:
			for i in range(len(vectors)):
				T_p1 = T_p1.col_join(vectors[i])

		for i in range(T.shape[0]):
			if T[i, j] == 0:
				T_p1 = T_p1.col_join(T[i, :])

		return T_p1

	def __next_tableau(self, T_i, j, n):

		S = self.__buildS(T_i, n)
		indices = self.__buildIndices(T_i, S, j)
		vectors = self.__buildVectors(T_i, j, indices)
		T_ip1 = self.__buildTp1(T_i, j, vectors)

		return T_ip1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


class EventAssignedVariable(object):

	def __init__(self, model):

		self.__model = model

		self.__hasEventAssignment = False
		self.__hasEventAssignmentBy = []

	# we need both add and remove implemented for shifts,
	# since we might have to delete some of them after having created them
	def addEventAssignmentBy(self, event, shift=0):
		self.__hasEventAssignmentBy.append((event.objId + shift))
		self.__hasEventAssignment = True

	def removeEventAssignmentBy(self, event, shift=0):
		self.__hasEventAssignmentBy.remove((event.objId + shift))
		self.__hasEventAssignment = (len(self.__hasEventAssignmentBy) > 0)

	def hasEventAssignment(self):
		return self.__hasEventAssignment

	def hasEventAssignmentBy(self):
		return [self.__model.listOfEvents[event] for event in self.__hasEventAssignmentBy]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.Simulation import Simulation

from libsignetsim.simulation.TimeseriesSimulation import TimeseriesSimulation

from libsignetsim.settings.Settings import Settings
from numpy import linspace

class UniformTimeCourse(Simulation):

	def __init__(self, document):

		Simulation.__init__(self, document)

		self.__document = document
		self.__initialTime = None
		self.__outputStartTime = None
		self.__outputEndTime = None
		self.__numberOfPoints = None

	def readSedml(self, simulation, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		Simulation.readSedml(self, simulation, level, version)

		if simulation.isSetInitialTime():
			self.__initialTime = simulation.getInitialTime()

		if simulation.isSetOutputStartTime():
			self.__outputStartTime = simulation.getOutputStartTime()

		if simulation.isSetOutputEndTime():
			self.__outputEndTime = simulation.getOutputEndTime()

		if simulation.isSetNumberOfPoints():
			self.__numberOfPoints = simulation.getNumberOfPoints()

	def writeSedml(self, simulation, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		Simulation.writeSedml(self, simulation, level,version)

		if self.__initialTime is not None:
			simulation.setInitialTime(self.__initialTime)

		if self.__outputStartTime is not None:
			simulation.setOutputStartTime(self.__outputStartTime)

		if self.__outputEndTime is not None:
			simulation.setOutputEndTime(self.__outputEndTime)

		if self.__numberOfPoints is not None:
			simulation.setNumberOfPoints(self.__numberOfPoints)


	def getInitialTime(self):
		return self.__initialTime

	def getOutputStartTime(self):
		return self.__outputStartTime

	def getOutputEndTime(self):
		return self.__outputEndTime

	def getNumberOfPoints(self):
		return self.__numberOfPoints

	def run(self, sbmlModel, timeout=None):
		samples = [float(value) for value in linspace(self.__outputStartTime, self.__outputEndTime, self.__numberOfPoints+1)]

		t_simulation = TimeseriesSimulation(
			list_of_models=[sbmlModel],
			time_min=self.__initialTime,
			list_samples=samples,
			abs_tol=Simulation.getAlgorithm(self).getAbsTol(),
			rel_tol=Simulation.getAlgorithm(self).getRelTol(),
		)
		t_simulation.run(timeout=timeout)
		return t_simulation

	def getSimulationObject(self):
		return self.__simulationObject

	def setInitialTime(self, initial_time):
		self.__initialTime = initial_time

	def setOutputStartTime(self, output_start_time):
		self.__outputStartTime = output_start_time

	def setOutputEndTime(self, output_end_time):
		self.__outputEndTime = output_end_time

	def setNumberOfPoints(self, number_of_points):
		self.__numberOfPoints = number_of_points



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.sympy_shortcuts import  (
	SympySymbol, SympyInteger, SympyFloat, SympyRational, SympyAtom,
	SympyOne, SympyNegOne, SympyZero, SympyPi, SympyE, SympyExp1, SympyHalf,
	SympyInf, SympyNan, SympyAdd, SympyMul, SympyPow,
	SympyFunction, SympyUndefinedFunction, SympyLambda, SympyDerivative,
	SympyCeiling, SympyFloor, SympyAbs, SympyLog, SympyExp, SympyPiecewise,
	SympyFactorial, SympyRoot, SympyAcos, SympyAsin, SympyAtan, SympyAcosh,
	SympyAsinh, SympyAtanh, SympyCos, SympySin, SympyTan, SympyAcot,
	SympyAcoth, SympyCosh, SympySinh, SympyTanh, SympySec, SympyCsc,
	SympyCot, SympyCoth, SympyAcsc, SympyAsec,
	SympyEqual, SympyUnequal, SympyGreaterThan, SympyLessThan,
	SympyStrictGreaterThan, SympyStrictLessThan,
	SympyAnd, SympyOr, SympyXor, SympyNot, SympyTrue, SympyFalse,
	SympyMax, SympyMin)


class MathSymbol(MathFormula):

	def __init__(self, model, variable, is_from_reaction=None):

		self.__model = model
		self.__variable = variable
		MathFormula.__init__(self, model, MathFormula.MATH_VARIABLE, isFromReaction=is_from_reaction)

	def readSbml(self, symbol, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		self.setInternalMathFormula(SympySymbol(symbol))

	# def setInternalVariable(self, internal_variable):
	# 	MathFormula.setInternalMathFormula(self, internal_variable)

	def getInternalMathFormula(self, rawFormula=False, developped=False):

		from libsignetsim.model.math.MathVariable import MathVariable

		# The if type(var) test is to check is the variable is a copied, simple math variable,
		# or if it's a more complex type, like species or parameter, which inherit from MathVariable
		# If it's just a math variable, if has no method isConcentration()
		if (type(self.__variable) == MathVariable
			or not self.__variable.isConcentration()
			or (self.__variable.isConcentration() and not rawFormula)):
			return MathFormula.getInternalMathFormula(self)
		else:

			if not developped:
				return SympySymbol("_speciesForcedConcentration_%s_" % MathFormula.getInternalMathFormula(self).name)
			else:
				return MathFormula.getInternalMathFormula(self)/self.__variable.getCompartment().symbol.getInternalMathFormula(rawFormula=rawFormula)


	def getSymbol(self):
		return MathFormula.getInternalMathFormula(self)

	# def getMathFormulaDerivative(self, math_type):
	#
	# 	if math_type in [MathFormula.MATH_INTERNAL, MathFormula.MATH_DEVINTERNAL, MathFormula.MATH_FINALINTERNAL]:
	# 		f = MathFormula.getMathFormula(self, math_type)
	# 		return SympyDerivative(f, MathFormula.t)


	def getDerivative(self):

		f = MathFormula.getInternalMathFormula(self)
		t_formula = MathFormula(self.__model, MathFormula.MATH_VARIABLE)
		t_formula.setInternalMathFormula(SympyDerivative(f, MathFormula.t))
		return t_formula


	# def getFinalMathFormula(self, forcedConcentration=False):
	# 	if self.isFromReaction is not None:
	# 		t_formula = MathFormula.getInternalMathFormula(self)
	# 		t_math = MathFormula(self.__model)
	# 		t_symbol = MathFormula.getInternalMathFormula(self)
	# 		t_new_symbol = SympySymbol(MathFormula.getSbmlMathFormula(self).getName())
	# 		t_math.setInternalMathFormula(t_formula.subs({t_symbol: t_new_symbol}))
	# 		return t_math.getFinalMathFormula(forcedConcentration=True)
	# 	else:
	# 		return MathFormula.getFinalMathFormula(self,forcedConcentration=True)

	# def getCMathFormula(self):
	# 	return self.writeCCode(MathFormula.getInternalMathFormula(self))

	def getPrettyPrintMathFormula(self, rawFormula=False):
		return str(self.getInternalMathFormula(rawFormula=rawFormula))

	# def renameSbmlId(self, old_sbml_id, new_sbml_id):


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.Model import Model
from libsignetsim.settings.Settings import Settings


class ListOfModels(ListOf):

	def __init__(self, document):

		ListOf.__init__(self, document)

		self.__document = document
		self.__modelCounter = 0

	def new(self, model_id=None):

		t_model_id = model_id
		if t_model_id is None:
			t_model_id = "model_%d" % self.__modelCounter

		model = Model(self.__document)
		model.setId(t_model_id)
		ListOf.append(self, model)
		self.__modelCounter += 1
		return model

	def createModel(self, model_id=None):
		return self.new(model_id)

	def readSedml(self, list_of_models, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_models, level, version)

		for t_model in list_of_models:
			model = Model(self.__document)
			model.readSedml(t_model, level, version)
			ListOf.append(self, model)
			self.__modelCounter += 1

	def writeSedml(self, list_of_models, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_models, level, version)

		for t_model in self:
			model = list_of_models.createModel()
			t_model.writeSedml(model, level, version)

	def getModelByReference(self, model_reference):

		for model in self:
			if model.getId() == model_reference:
				return model


	def getSbmlModels(self):

		models = []
		for model in self:
			models.append(model.getSbmlModel())

		return models

	def getSbmlModelByReference(self, model_reference):

		for model in self:
			if model.getId() == model_reference:
				return model.getSbmlModel()

	def makeRelativePaths(self, path):
		for model in self:
			model.makeRelativePath(path)

	def writeSbmlModelsToPath(self, path):

		for model in self:
			model.writeSbmlModelToPath(path)

	def removePaths(self):

		for model in self:
			model.removePaths()

	def makeLocalSources(self):
		filenames = []
		for model in self:
			t_filename = model.makeLocalSource()
			if t_filename is not None:
				filenames.append(t_filename)

		return filenames